from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.html_widgets import TwoStateWidget
from mayan.apps.common.menus import (
    menu_object, menu_secondary, menu_tools, menu_topbar, menu_user
)
from mayan.apps.navigation.classes import SourceColumn

from .html_widgets import (
    ObjectLinkWidget, widget_event_actor_link, widget_event_type_link
)
from .links import (
    link_current_user_events, link_event_types_subscriptions_list,
    link_events_list, link_notification_mark_read,
    link_notification_mark_read_all, link_user_notifications_list,
)


class EventsApp(MayanAppConfig):
    app_namespace = 'events'
    app_url = 'events'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.events'
    verbose_name = _('Events')

    def ready(self):
        super(EventsApp, self).ready()
        Action = apps.get_model(app_label='actstream', model_name='Action')
        Notification = self.get_model(model_name='Notification')
        StoredEventType = self.get_model(model_name='StoredEventType')

        # Add labels to Action model, they are not marked translatable in the
        # upstream package.
        SourceColumn(
            attribute='timestamp', is_identifier=True,
            is_sortable=True, label=_('Date and time'), source=Action
        )
        SourceColumn(
            func=widget_event_actor_link, label=_('Actor'), source=Action
        )
        SourceColumn(
            func=widget_event_type_link, label=_('Event'), source=Action
        )
        SourceColumn(
            attribute='target', label=_('Target'), source=Action,
            widget=ObjectLinkWidget
        )
        SourceColumn(
            attribute='action_object', label=_('Action object'), source=Action,
            widget=ObjectLinkWidget
        )

        SourceColumn(
            source=StoredEventType, label=_('Namespace'), attribute='namespace'
        )
        SourceColumn(
            source=StoredEventType, label=_('Label'), attribute='label'
        )

        SourceColumn(
            attribute='action__timestamp', is_identifier=True,
            is_sortable=True, label=_('Date and time'), source=Notification
        )
        SourceColumn(
            func=widget_event_actor_link, label=_('Actor'),
            kwargs={'attribute': 'action'}, source=Notification
        )
        SourceColumn(
            func=widget_event_type_link, label=_('Event'),
            kwargs={'attribute': 'action'}, source=Notification
        )
        SourceColumn(
            attribute='action.target', label=_('Target'), source=Notification,
            widget=ObjectLinkWidget
        )

        SourceColumn(
            attribute='read', is_sortable=True, label=_('Seen'),
            source=Notification, widget=TwoStateWidget
        )

        menu_topbar.bind_links(
            links=(link_user_notifications_list,), position=99
        )
        menu_object.bind_links(
            links=(link_notification_mark_read,), sources=(Notification,)
        )
        menu_secondary.bind_links(
            links=(link_notification_mark_read_all,),
            sources=('events:user_notifications_list',)
        )
        menu_tools.bind_links(links=(link_events_list,))
        menu_user.bind_links(
            links=(
                link_event_types_subscriptions_list, link_current_user_events
            ), position=50
        )



from __future__ import unicode_literals

from rest_framework import serializers
from rest_framework.reverse import reverse

from mayan.apps.documents.serializers import DocumentSerializer
from mayan.apps.user_management.serializers import UserSerializer

from .models import Comment


class CommentSerializer(serializers.HyperlinkedModelSerializer):
    document = DocumentSerializer(read_only=True)
    document_comments_url = serializers.SerializerMethodField()
    url = serializers.SerializerMethodField()
    user = UserSerializer(read_only=True)

    class Meta:
        fields = (
            'comment', 'document', 'document_comments_url', 'id',
            'submit_date', 'url', 'user'
        )
        model = Comment

    def get_document_comments_url(self, instance):
        return reverse(
            'rest_api:comment-list', args=(
                instance.document.pk,
            ), request=self.context['request'], format=self.context['format']
        )

    def get_url(self, instance):
        return reverse(
            'rest_api:comment-detail', args=(
                instance.document.pk, instance.pk
            ), request=self.context['request'], format=self.context['format']
        )


class WritableCommentSerializer(serializers.ModelSerializer):
    document = DocumentSerializer(read_only=True)
    document_comments_url = serializers.SerializerMethodField()
    url = serializers.SerializerMethodField()
    user = UserSerializer(read_only=True)

    class Meta:
        fields = (
            'comment', 'document', 'document_comments_url', 'id',
            'submit_date', 'url', 'user'
        )
        model = Comment
        read_only_fields = ('document',)

    def create(self, validated_data):
        validated_data['document'] = self.context['document']
        validated_data['user'] = self.context['request'].user
        return super(WritableCommentSerializer, self).create(validated_data)

    def get_document_comments_url(self, instance):
        return reverse(
            'rest_api:comment-list', args=(
                instance.document.pk,
            ), request=self.context['request'], format=self.context['format']
        )

    def get_url(self, instance):
        return reverse(
            'rest_api:comment-detail', args=(
                instance.document.pk, instance.pk
            ), request=self.context['request'], format=self.context['format']
        )



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0020_auto_20181128_0752'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailbasemodel',
            name='metadata_attachment_name',
            field=models.CharField(
                default='metadata.yaml', help_text='Name of the attachment '
                'that will contains the metadata type names and value '
                'pairs to be assigned to the rest of the downloaded '
                'attachments.', max_length=128,
                verbose_name='Metadata attachment name'
            ),
        ),
    ]



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

from .parsers import MetadataParser
from .validators import MetadataValidator

namespace = Namespace(label=_('Metadata'), name='metadata')

setting_available_validators = namespace.add_setting(
    global_name='METADATA_AVAILABLE_VALIDATORS',
    default=MetadataValidator.get_import_paths()
)
setting_available_parsers = namespace.add_setting(
    global_name='METADATA_AVAILABLE_PARSERS',
    default=MetadataParser.get_import_paths()
)



from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from .tasks import (
    task_delete_empty, task_index_document, task_remove_document
)


def handler_create_default_document_index(sender, **kwargs):
    DocumentType = apps.get_model(
        app_label='documents', model_name='DocumentType'
    )
    Index = apps.get_model(
        app_label='document_indexing', model_name='Index'
    )

    index = Index.objects.create(
        label=_('Creation date'), slug='creation_date'
    )
    for document_type in DocumentType.objects.all():
        index.document_types.add(document_type)

    root_template_node = index.template_root
    node = root_template_node.get_children().create(
        expression='{{ document.date_added|date:"Y" }}', index=index,
        parent=root_template_node
    )
    node.get_children().create(
        expression='{{ document.date_added|date:"m" }}',
        index=index, link_documents=True, parent=node
    )


def handler_delete_empty(sender, **kwargs):
    task_delete_empty.apply_async()


def handler_index_document(sender, **kwargs):
    task_index_document.apply_async(
        kwargs=dict(document_id=kwargs['instance'].pk)
    )


def handler_post_save_index_document(sender, **kwargs):
    """
    Reindex documents when they get edited. For indexing documents
    when they are first created the handler_index_document is called
    from the custom post_document_created signal.
    """
    if not kwargs['created']:
        task_index_document.apply_async(
            kwargs=dict(document_id=kwargs['instance'].pk)
        )


def handler_remove_document(sender, **kwargs):
    task_remove_document.apply_async(
        kwargs=dict(document_id=kwargs['instance'].pk)
    )



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APIDocumentTagView, APIDocumentTagListView, APITagDocumentListView,
    APITagListView, APITagView
)
from .views import (
    DocumentTagListView, TagAttachActionView, TagCreateView,
    TagDeleteActionView, TagEditView, TagListView, TagRemoveActionView,
    TagDocumentListView
)

urlpatterns = [
    url(regex=r'^tags/$', view=TagListView.as_view(), name='tag_list'),
    url(regex=r'^tags/create/$', view=TagCreateView.as_view(), name='tag_create'),
    url(
        regex=r'^tags/(?P<pk>\d+)/delete/$', view=TagDeleteActionView.as_view(),
        name='tag_delete'
    ),
    url(
        regex=r'^tags/(?P<pk>\d+)/edit/$', view=TagEditView.as_view(),
        name='tag_edit'
    ),
    url(
        regex=r'^tags/(?P<pk>\d+)/documents/$', view=TagDocumentListView.as_view(),
        name='tag_document_list'
    ),
    url(
        regex=r'^tags/multiple/delete/$', view=TagDeleteActionView.as_view(),
        name='tag_multiple_delete'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/tags/remove/$',
        view=TagRemoveActionView.as_view(),
        name='single_document_multiple_tag_remove'
    ),
    url(
        regex=r'^documents/multiple/tags/remove/$',
        view=TagRemoveActionView.as_view(),
        name='multiple_documents_selection_tag_remove'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/tags/attach/$',
        view=TagAttachActionView.as_view(), name='tag_attach'
    ),
    url(
        regex=r'^documents/multiple/tags/attach/$',
        view=TagAttachActionView.as_view(), name='multiple_documents_tag_attach'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/tags/$',
        view=DocumentTagListView.as_view(), name='document_tag_list'
    ),
]

api_urls = [
    url(
        regex=r'^tags/(?P<pk>[0-9]+)/documents/$',
        view=APITagDocumentListView.as_view(), name='tag-document-list'
    ),
    url(
        regex=r'^tags/(?P<pk>[0-9]+)/$', view=APITagView.as_view(),
        name='tag-detail'
    ),
    url(regex=r'^tags/$', view=APITagListView.as_view(), name='tag-list'),
    url(
        regex=r'^documents/(?P<document_pk>[0-9]+)/tags/$',
        view=APIDocumentTagListView.as_view(), name='document-tag-list'
    ),
    url(
        regex=r'^documents/(?P<document_pk>[0-9]+)/tags/(?P<pk>[0-9]+)/$',
        view=APIDocumentTagView.as_view(), name='document-tag-detail'
    ),
]



from __future__ import unicode_literals

import logging

from django.apps import apps

from mayan.apps.lock_manager.exceptions import LockError
from mayan.apps.lock_manager.runtime import locking_backend
from mayan.celery import app

from .literals import LOCK_EXPIRE

from .classes import FileMetadataDriver

logger = logging.getLogger(__name__)


@app.task(ignore_result=True)
def task_process_document_version(document_version_id):
    DocumentVersion = apps.get_model(
        app_label='documents', model_name='DocumentVersion'
    )

    document_version = DocumentVersion.objects.get(pk=document_version_id)

    lock_id = 'task_process_document_version-%d' % document_version_id
    try:
        logger.debug('trying to acquire lock: %s', lock_id)
        # Acquire lock to avoid processing the same document version more
        # than once concurrently
        lock = locking_backend.acquire_lock(name=lock_id, timeout=LOCK_EXPIRE)
        logger.debug('acquired lock: %s', lock_id)
    except LockError:
        logger.debug('unable to obtain lock: %s' % lock_id)
    else:
        FileMetadataDriver.process_document_version(
            document_version=document_version
        )
        lock.release()



from __future__ import absolute_import, unicode_literals

from django.core.files import File

from mayan.apps.django_gpg.models import Key
from mayan.apps.django_gpg.tests.literals import TEST_KEY_PRIVATE_PASSPHRASE

from ..models import DetachedSignature

from .literals import TEST_KEY_FILE_PATH, TEST_SIGNATURE_FILE_PATH


class DetachedSignatureAPIViewTestMixin(object):
    def _request_test_document_signature_detached_create_view(self):
        with open(TEST_SIGNATURE_FILE_PATH, mode='rb') as file_object:
            return self.post(
                viewname='rest_api:document-version-signature-detached-list',
                kwargs={
                    'document_id': self.test_document.pk,
                    'document_version_id': self.test_document_version.pk
                }, data={'signature_file': file_object}
            )

    def _request_test_document_signature_detached_delete_view(self):
        return self.delete(
            viewname='rest_api:detachedsignature-detail',
            kwargs={
                'document_id': self.test_document.pk,
                'document_version_id': self.test_document_version.pk,
                'detached_signature_id': self.test_document_version.signatures.first().pk
            }
        )

    def _request_test_document_signature_detached_detail_view(self):
        return self.get(
            viewname='rest_api:detachedsignature-detail',
            kwargs={
                'document_id': self.test_document.pk,
                'document_version_id': self.test_document_version.pk,
                'detached_signature_id': self.test_document_version.signatures.first().pk
            }
        )

    def _request_test_document_signature_detached_list_view(self):
        return self.get(
            viewname='rest_api:document-version-signature-detached-list',
            kwargs={
                'document_id': self.test_document.pk,
                'document_version_id': self.test_document_version.pk
            }
        )

    def _request_test_document_signature_detached_sign_view(self):
        return self.post(
            viewname='rest_api:document-version-signature-detached-sign',
            kwargs={
                'document_id': self.test_document.pk,
                'document_version_id': self.test_document_version.pk
            }, data={
                'key_id': self.test_key_private.key_id,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )


class DetachedSignatureViewTestMixin(object):
    def _request_test_document_version_signature_create_view(self):
        return self.post(
            viewname='signatures:document_version_signature_detached_create',
            kwargs={'pk': self.test_document_version.pk}, data={
                'key': self.test_key_private.pk,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )

    def _request_test_document_version_signature_download_view(self):
        return self.get(
            viewname='signatures:document_version_signature_download',
            kwargs={'pk': self.test_signature.pk}
        )

    def _request_test_document_version_signature_upload_view(self):
        with open(TEST_SIGNATURE_FILE_PATH, mode='rb') as file_object:
            return self.post(
                viewname='signatures:document_version_signature_upload',
                kwargs={'pk': self.test_document.latest_version.pk},
                data={'signature_file': file_object}
            )


class EmbeddedSignatureAPIViewTestMixin(object):
    def _request_test_document_signature_embedded_detail_view(self):
        return self.get(
            viewname='rest_api:embeddedsignature-detail',
            kwargs={
                'document_id': self.test_document.pk,
                'document_version_id': self.test_document_version.pk,
                'embedded_signature_id': self.test_document_version.signatures.first().pk
            }
        )

    def _request_test_document_signature_embedded_list_view(self):
        return self.get(
            viewname='rest_api:document-version-signature-embedded-list',
            kwargs={
                'document_id': self.test_document.pk,
                'document_version_id': self.test_document_version.pk
            }
        )

    def _request_test_document_signature_embedded_sign_view(self):
        return self.post(
            viewname='rest_api:document-version-signature-embedded-sign',
            kwargs={
                'document_id': self.test_document.pk,
                'document_version_id': self.test_document_version.pk
            }, data={
                'key_id': self.test_key_private.key_id,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )


class EmbeddedSignatureViewTestMixin(object):
    def _request_test_document_version_signature_create_view(self):
        return self.post(
            viewname='signatures:document_version_signature_embedded_create',
            kwargs={'pk': self.test_document_version.pk}, data={
                'key': self.test_key_private.pk,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )


class SignatureTestMixin(object):
    def _upload_test_detached_signature(self):
        with open(TEST_SIGNATURE_FILE_PATH, mode='rb') as file_object:
            self.test_signature = DetachedSignature.objects.create(
                document_version=self.test_document.latest_version,
                signature_file=File(file_object)
            )

    def _create_test_key_public(self):
        with open(TEST_KEY_FILE_PATH, mode='rb') as file_object:
            self.test_key_public = Key.objects.create(
                key_data=file_object.read()
            )


class SignatureViewTestMixin(object):
    def _request_test_document_version_signature_delete_view(self):
        return self.post(
            viewname='signatures:document_version_signature_delete',
            kwargs={'pk': self.test_signature.pk}
        )

    def _request_test_document_version_signature_details_view(self):
        return self.get(
            viewname='signatures:document_version_signature_details',
            kwargs={'pk': self.test_signature.pk}
        )

    def _request_test_document_version_signature_list_view(self, document):
        return self.get(
            viewname='signatures:document_version_signature_list',
            kwargs={'pk': self.test_document.latest_version.pk}
        )

    def _request_all_test_document_version_signature_verify_view(self):
        return self.post(
            viewname='signatures:all_document_version_signature_verify'
        )



from __future__ import unicode_literals

CONFIGURATION_FILENAME = 'config.yml'
CONFIGURATION_LAST_GOOD_FILENAME = 'config_backup.yml'
NAMESPACE_VERSION_INITIAL = '0001'
SMART_SETTINGS_NAMESPACES_NAME = 'SMART_SETTINGS_NAMESPACES'



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .permissions import (
    permission_mailing_link, permission_mailing_send_document,
    permission_user_mailer_create, permission_user_mailer_delete,
    permission_user_mailer_edit, permission_user_mailer_use,
    permission_user_mailer_view, permission_view_error_log
)

link_send_document = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.mailer.icons.icon_document_send',
    permissions=(permission_mailing_send_document,),
    text=_('Email document'), view='mailer:send_document'
)
link_send_document_link = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.mailer.icons.icon_document_send_link',
    permissions=(permission_mailing_link,),
    text=_('Email link'), view='mailer:send_document_link'
)
link_send_multiple_document = Link(
    icon_class_path='mayan.apps.mailer.icons.icon_document_send',
    text=_('Email document'), view='mailer:send_multiple_document'
)
link_send_multiple_document_link = Link(
    icon_class_path='mayan.apps.mailer.icons.icon_document_send_link',
    text=_('Email link'), view='mailer:send_multiple_document_link'
)
link_system_mailer_error_log = Link(
    icon_class_path='mayan.apps.mailer.icons.icon_system_mailer_error_log',
    permissions=(permission_view_error_log,),
    text=_('System mailer error log'), view='mailer:system_mailer_error_log',
)
link_user_mailer_create = Link(
    icon_class_path='mayan.apps.mailer.icons.icon_user_mailer_create',
    permissions=(permission_user_mailer_create,),
    text=_('Create mailing profile '), view='mailer:user_mailer_backend_selection',
)
link_user_mailer_delete = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.mailer.icons.icon_user_mailer_delete',
    permissions=(permission_user_mailer_delete,),
    tags='dangerous', text=_('Delete'), view='mailer:user_mailer_delete',
)
link_user_mailer_edit = Link(
    args='object.pk',
    icon_class_path='mayan.apps.mailer.icons.icon_user_mailer_edit',
    permissions=(permission_user_mailer_edit,),
    text=_('Edit'), view='mailer:user_mailer_edit',
)
link_user_mailer_log_list = Link(
    args='object.pk', permissions=(permission_user_mailer_view,),
    text=_('Log'), view='mailer:user_mailer_log',
)
link_user_mailer_list = Link(
    icon_class_path='mayan.apps.mailer.icons.icon_user_mailer_list',
    permissions=(permission_user_mailer_view,),
    text=_('Mailing profiles list'), view='mailer:user_mailer_list',
)
link_user_mailer_setup = Link(
    icon_class_path='mayan.apps.mailer.icons.icon_user_mailer_setup',
    permissions=(permission_user_mailer_view,),
    text=_('Mailing profiles'), view='mailer:user_mailer_list',
)
link_user_mailer_test = Link(
    args='object.pk',
    icon_class_path='mayan.apps.mailer.icons.icon_user_mailer_test',
    permissions=(permission_user_mailer_use,),
    text=_('Test'), view='mailer:user_mailer_test',
)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MessageOfTheDay',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'label', models.CharField(
                        max_length=32, verbose_name='Label'
                    )
                ),
                (
                    'message', models.TextField(
                        verbose_name='Message', blank=True
                    )
                ),
                (
                    'enabled', models.BooleanField(
                        default=True, verbose_name='Enabled'
                    )
                ),
                (
                    'start_datetime', models.DateTimeField(
                        verbose_name='Start date time', blank=True
                    )
                ),
                (
                    'end_datetime', models.DateTimeField(
                        verbose_name='End date time', blank=True
                    )
                ),
            ],
            options={
                'verbose_name': 'Message of the day',
                'verbose_name_plural': 'Messages of the day',
            },
        ),
    ]



from __future__ import unicode_literals

from mayan.apps.storage.utils import get_storage_subclass

from .settings import (
    setting_staging_file_image_cache_storage,
    setting_staging_file_image_cache_storage_arguments,
)

storage_staging_file_image_cache = get_storage_subclass(
    dotted_path=setting_staging_file_image_cache_storage.value
)(**setting_staging_file_image_cache_storage_arguments.value)



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0019_auto_20150714_2232'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='description',
            field=models.TextField(
                default='', null=True, verbose_name='Description', blank=True
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='document',
            name='label',
            field=models.CharField(
                default='', max_length=255, blank=True,
                help_text='The name of the document', null=True,
                verbose_name='Label', db_index=True
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='document',
            name='language',
            field=models.CharField(
                default='eng', choices=[
                    ['aar', 'Afar'],
                    ['abk', 'Abkhazian'],
                    ['ace', 'Achinese'],
                    ['ach', 'Acoli'],
                    ['ada', 'Adangme'],
                    ['ady', 'Adyghe; Adygei'],
                    ['afa', 'Afro-Asiatic languages'],
                    ['afh', 'Afrihili'],
                    ['afr', 'Afrikaans'],
                    ['ain', 'Ainu'],
                    ['aka', 'Akan'],
                    ['akk', 'Akkadian'],
                    ['alb', 'Albanian'],
                    ['ale', 'Aleut'],
                    ['alg', 'Algonquian languages'],
                    ['alt', 'Southern Altai'],
                    ['amh', 'Amharic'],
                    ['ang', 'English, Old (ca. 450-1100)'],
                    ['anp', 'Angika'],
                    ['apa', 'Apache languages'],
                    ['ara', 'Arabic'],
                    [
                        b'arc', 'Official Aramaic (700-300 BCE); Imperial '
                        'Aramaic (700-300 BCE)'
                    ],
                    ['arg', 'Aragonese'],
                    ['arm', 'Armenian'],
                    ['arn', 'Mapudungun; Mapuche'],
                    ['arp', 'Arapaho'],
                    ['art', 'Artificial languages'],
                    ['arw', 'Arawak'],
                    ['asm', 'Assamese'],
                    ['ast', 'Asturian; Bable; Leonese; Asturleonese'],
                    ['ath', 'Athapascan languages'],
                    ['aus', 'Australian languages'],
                    ['ava', 'Avaric'],
                    ['ave', 'Avestan'],
                    ['awa', 'Awadhi'],
                    ['aym', 'Aymara'],
                    ['aze', 'Azerbaijani'],
                    ['bad', 'Banda languages'],
                    ['bai', 'Bamileke languages'],
                    ['bak', 'Bashkir'],
                    ['bal', 'Baluchi'],
                    ['bam', 'Bambara'],
                    ['ban', 'Balinese'],
                    ['baq', 'Basque'],
                    ['bas', 'Basa'],
                    ['bat', 'Baltic languages'],
                    ['bej', 'Beja; Bedawiyet'],
                    ['bel', 'Belarusian'],
                    ['bem', 'Bemba'],
                    ['ben', 'Bengali'],
                    ['ber', 'Berber languages'],
                    ['bho', 'Bhojpuri'],
                    ['bih', 'Bihari languages'],
                    ['bik', 'Bikol'],
                    ['bin', 'Bini; Edo'],
                    ['bis', 'Bislama'],
                    ['bla', 'Siksika'],
                    ['bnt', 'Bantu languages'],
                    ['bos', 'Bosnian'],
                    ['bra', 'Braj'],
                    ['bre', 'Breton'],
                    ['btk', 'Batak languages'],
                    ['bua', 'Buriat'],
                    ['bug', 'Buginese'],
                    ['bul', 'Bulgarian'],
                    ['bur', 'Burmese'],
                    ['byn', 'Blin; Bilin'],
                    ['cad', 'Caddo'],
                    ['cai', 'Central American Indian languages'],
                    ['car', 'Galibi Carib'],
                    ['cat', 'Catalan; Valencian'],
                    ['cau', 'Caucasian languages'],
                    ['ceb', 'Cebuano'],
                    ['cel', 'Celtic languages'],
                    ['cha', 'Chamorro'],
                    ['chb', 'Chibcha'],
                    ['che', 'Chechen'],
                    ['chg', 'Chagatai'],
                    ['chi', 'Chinese'],
                    ['chk', 'Chuukese'],
                    ['chm', 'Mari'],
                    ['chn', 'Chinook jargon'],
                    ['cho', 'Choctaw'],
                    ['chp', 'Chipewyan; Dene Suline'],
                    ['chr', 'Cherokee'],
                    [
                        b'chu', 'Church Slavic; Old Slavonic; Church '
                        'Slavonic; Old Bulgarian; Old Church Slavonic'
                    ],
                    ['chv', 'Chuvash'],
                    ['chy', 'Cheyenne'],
                    ['cmc', 'Chamic languages'],
                    ['cop', 'Coptic'],
                    ['cor', 'Cornish'],
                    ['cos', 'Corsican'],
                    ['cpe', 'Creoles and pidgins, English based'],
                    ['cpf', 'Creoles and pidgins, French-based'],
                    ['cpp', 'Creoles and pidgins, Portuguese-based'],
                    ['cre', 'Cree'],
                    ['crh', 'Crimean Tatar; Crimean Turkish'],
                    ['crp', 'Creoles and pidgins'],
                    ['csb', 'Kashubian'],
                    ['cus', 'Cushitic languages'],
                    ['cze', 'Czech'],
                    ['dak', 'Dakota'],
                    ['dan', 'Danish'],
                    ['dar', 'Dargwa'],
                    ['day', 'Land Dayak languages'],
                    ['del', 'Delaware'],
                    ['den', 'Slave (Athapascan)'],
                    ['dgr', 'Dogrib'],
                    ['din', 'Dinka'],
                    ['div', 'Divehi; Dhivehi; Maldivian'],
                    ['doi', 'Dogri'],
                    ['dra', 'Dravidian languages'],
                    ['dsb', 'Lower Sorbian'],
                    ['dua', 'Duala'],
                    ['dum', 'Dutch, Middle (ca. 1050-1350)'],
                    ['dut', 'Dutch; Flemish'],
                    ['dyu', 'Dyula'],
                    ['dzo', 'Dzongkha'],
                    ['efi', 'Efik'],
                    ['egy', 'Egyptian (Ancient)'],
                    ['eka', 'Ekajuk'],
                    ['elx', 'Elamite'],
                    ['eng', 'English'],
                    ['enm', 'English, Middle (1100-1500)'],
                    ['epo', 'Esperanto'],
                    ['est', 'Estonian'],
                    ['ewe', 'Ewe'],
                    ['ewo', 'Ewondo'],
                    ['fan', 'Fang'],
                    ['fao', 'Faroese'],
                    ['fat', 'Fanti'],
                    ['fij', 'Fijian'],
                    ['fil', 'Filipino; Pilipino'],
                    ['fin', 'Finnish'],
                    ['fiu', 'Finno-Ugrian languages'],
                    ['fon', 'Fon'],
                    ['fre', 'French'],
                    ['frm', 'French, Middle (ca. 1400-1600)'],
                    ['fro', 'French, Old (842-ca. 1400)'],
                    ['frr', 'Northern Frisian'],
                    ['frs', 'Eastern Frisian'],
                    ['fry', 'Western Frisian'],
                    ['ful', 'Fulah'],
                    ['fur', 'Friulian'],
                    ['gaa', 'Ga'],
                    ['gay', 'Gayo'],
                    ['gba', 'Gbaya'],
                    ['gem', 'Germanic languages'],
                    ['geo', 'Georgian'],
                    ['ger', 'German'],
                    ['gez', 'Geez'],
                    ['gil', 'Gilbertese'],
                    ['gla', 'Gaelic; Scottish Gaelic'],
                    ['gle', 'Irish'],
                    ['glg', 'Galician'],
                    ['glv', 'Manx'],
                    ['gmh', 'German, Middle High (ca. 1050-1500)'],
                    ['goh', 'German, Old High (ca. 750-1050)'],
                    ['gon', 'Gondi'],
                    ['gor', 'Gorontalo'],
                    ['got', 'Gothic'],
                    ['grb', 'Grebo'],
                    ['grc', 'Greek, Ancient (to 1453)'],
                    ['gre', 'Greek, Modern (1453-)'],
                    ['grn', 'Guarani'],
                    ['gsw', 'Swiss German; Alemannic; Alsatian'],
                    ['guj', 'Gujarati'],
                    ['gwi', b"Gwich'in"],
                    ['hai', 'Haida'],
                    ['hat', 'Haitian; Haitian Creole'],
                    ['hau', 'Hausa'],
                    ['haw', 'Hawaiian'],
                    ['heb', 'Hebrew'],
                    ['her', 'Herero'],
                    ['hil', 'Hiligaynon'],
                    ['him', 'Himachali languages; Western Pahari languages'],
                    ['hin', 'Hindi'],
                    ['hit', 'Hittite'],
                    ['hmn', 'Hmong; Mong'],
                    ['hmo', 'Hiri Motu'],
                    ['hrv', 'Croatian'],
                    ['hsb', 'Upper Sorbian'],
                    ['hun', 'Hungarian'],
                    ['hup', 'Hupa'],
                    ['iba', 'Iban'],
                    ['ibo', 'Igbo'],
                    ['ice', 'Icelandic'],
                    ['ido', 'Ido'],
                    ['iii', 'Sichuan Yi; Nuosu'],
                    ['ijo', 'Ijo languages'],
                    ['iku', 'Inuktitut'],
                    ['ile', 'Interlingue; Occidental'],
                    ['ilo', 'Iloko'],
                    [
                        b'ina', 'Interlingua (International Auxiliary '
                        'Language Association)'
                    ],
                    ['inc', 'Indic languages'],
                    ['ind', 'Indonesian'],
                    ['ine', 'Indo-European languages'],
                    ['inh', 'Ingush'],
                    ['ipk', 'Inupiaq'],
                    ['ira', 'Iranian languages'],
                    ['iro', 'Iroquoian languages'],
                    ['ita', 'Italian'],
                    ['jav', 'Javanese'],
                    ['jbo', 'Lojban'],
                    ['jpn', 'Japanese'],
                    ['jpr', 'Judeo-Persian'],
                    ['jrb', 'Judeo-Arabic'],
                    ['kaa', 'Kara-Kalpak'],
                    ['kab', 'Kabyle'],
                    ['kac', 'Kachin; Jingpho'],
                    ['kal', 'Kalaallisut; Greenlandic'],
                    ['kam', 'Kamba'],
                    ['kan', 'Kannada'],
                    ['kar', 'Karen languages'],
                    ['kas', 'Kashmiri'],
                    ['kau', 'Kanuri'],
                    ['kaw', 'Kawi'],
                    ['kaz', 'Kazakh'],
                    ['kbd', 'Kabardian'],
                    ['kha', 'Khasi'],
                    ['khi', 'Khoisan languages'],
                    ['khm', 'Central Khmer'],
                    ['kho', 'Khotanese;Sakan'],
                    ['kik', 'Kikuyu; Gikuyu'],
                    ['kin', 'Kinyarwanda'],
                    ['kir', 'Kirghiz; Kyrgyz'],
                    ['kmb', 'Kimbundu'],
                    ['kok', 'Konkani'],
                    ['kom', 'Komi'],
                    ['kon', 'Kongo'],
                    ['kor', 'Korean'],
                    ['kos', 'Kosraean'],
                    ['kpe', 'Kpelle'],
                    ['krc', 'Karachay-Balkar'],
                    ['krl', 'Karelian'],
                    ['kro', 'Kru languages'],
                    ['kru', 'Kurukh'],
                    ['kua', 'Kuanyama; Kwanyama'],
                    ['kum', 'Kumyk'],
                    ['kur', 'Kurdish'],
                    ['kut', 'Kutenai'],
                    ['lad', 'Ladino'],
                    ['lah', 'Lahnda'],
                    ['lam', 'Lamba'],
                    ['lao', 'Lao'],
                    ['lat', 'Latin'],
                    ['lav', 'Latvian'],
                    ['lez', 'Lezghian'],
                    ['lim', 'Limburgan; Limburger; Limburgish'],
                    ['lin', 'Lingala'],
                    ['lit', 'Lithuanian'],
                    ['lol', 'Mongo'],
                    ['loz', 'Lozi'],
                    ['ltz', 'Luxembourgish; Letzeburgesch'],
                    ['lua', 'Luba-Lulua'],
                    ['lub', 'Luba-Katanga'],
                    ['lug', 'Ganda'],
                    ['lui', 'Luiseno'],
                    ['lun', 'Lunda'],
                    ['luo', 'Luo (Kenya and Tanzania)'],
                    ['lus', 'Lushai'],
                    ['mac', 'Macedonian'],
                    ['mad', 'Madurese'],
                    ['mag', 'Magahi'],
                    ['mah', 'Marshallese'],
                    ['mai', 'Maithili'],
                    ['mak', 'Makasar'],
                    ['mal', 'Malayalam'],
                    ['man', 'Mandingo'],
                    ['mao', 'Maori'],
                    ['map', 'Austronesian languages'],
                    ['mar', 'Marathi'],
                    ['mas', 'Masai'],
                    ['may', 'Malay'],
                    ['mdf', 'Moksha'],
                    ['mdr', 'Mandar'],
                    ['men', 'Mende'],
                    ['mga', 'Irish, Middle (900-1200)'],
                    ['mic', b"Mi'kmaq; Micmac"],
                    ['min', 'Minangkabau'],
                    ['mis', 'Uncoded languages'],
                    ['mkh', 'Mon-Khmer languages'],
                    ['mlg', 'Malagasy'],
                    ['mlt', 'Maltese'],
                    ['mnc', 'Manchu'],
                    ['mni', 'Manipuri'],
                    ['mno', 'Manobo languages'],
                    ['moh', 'Mohawk'],
                    ['mol', 'Moldavian; Moldovan'],
                    ['mon', 'Mongolian'],
                    ['mos', 'Mossi'],
                    ['mul', 'Multiple languages'],
                    ['mun', 'Munda languages'],
                    ['mus', 'Creek'],
                    ['mwl', 'Mirandese'],
                    ['mwr', 'Marwari'],
                    ['myn', 'Mayan languages'],
                    ['myv', 'Erzya'],
                    ['nah', 'Nahuatl languages'],
                    ['nai', 'North American Indian languages'],
                    ['nap', 'Neapolitan'],
                    ['nau', 'Nauru'],
                    ['nav', 'Navajo; Navaho'],
                    ['nbl', 'Ndebele, South; South Ndebele'],
                    ['nde', 'Ndebele, North; North Ndebele'],
                    ['ndo', 'Ndonga'],
                    [
                        b'nds', 'Low German; Low Saxon; German, Low; Saxon, '
                        'Low'
                    ],
                    ['nep', 'Nepali'],
                    ['new', 'Nepal Bhasa; Newari'],
                    ['nia', 'Nias'],
                    ['nic', 'Niger-Kordofanian languages'],
                    ['niu', 'Niuean'],
                    ['nno', 'Norwegian Nynorsk; Nynorsk, Norwegian'],
                    ['nob', 'Bokm\xe5l, Norwegian; Norwegian Bokm\xe5l'],
                    ['nog', 'Nogai'],
                    ['non', 'Norse, Old'],
                    ['nor', 'Norwegian'],
                    ['nqo', b"N'Ko"],
                    ['nso', 'Pedi; Sepedi; Northern Sotho'],
                    ['nub', 'Nubian languages'],
                    [
                        b'nwc', 'Classical Newari; Old Newari; Classical '
                        'Nepal Bhasa'
                    ],
                    ['nya', 'Chichewa; Chewa; Nyanja'],
                    ['nym', 'Nyamwezi'],
                    ['nyn', 'Nyankole'],
                    ['nyo', 'Nyoro'],
                    ['nzi', 'Nzima'],
                    ['oci', 'Occitan (post 1500)'],
                    ['oji', 'Ojibwa'],
                    ['ori', 'Oriya'],
                    ['orm', 'Oromo'],
                    ['osa', 'Osage'],
                    ['oss', 'Ossetian; Ossetic'],
                    ['ota', 'Turkish, Ottoman (1500-1928)'],
                    ['oto', 'Otomian languages'],
                    ['paa', 'Papuan languages'],
                    ['pag', 'Pangasinan'],
                    ['pal', 'Pahlavi'],
                    ['pam', 'Pampanga; Kapampangan'],
                    ['pan', 'Panjabi; Punjabi'],
                    ['pap', 'Papiamento'],
                    ['pau', 'Palauan'],
                    ['peo', 'Persian, Old (ca. 600-400 B.C.)'],
                    ['per', 'Persian'],
                    ['phi', 'Philippine languages'],
                    ['phn', 'Phoenician'],
                    ['pli', 'Pali'],
                    ['pol', 'Polish'],
                    ['pon', 'Pohnpeian'],
                    ['por', 'Portuguese'],
                    ['pra', 'Prakrit languages'],
                    [
                        b'pro', 'Proven\xe7al, Old (to 1500); Occitan, '
                        'Old (to 1500)'
                    ],
                    ['pus', 'Pushto; Pashto'],
                    ['qaa-qtz', 'Reserved for local use'],
                    ['que', 'Quechua'],
                    ['raj', 'Rajasthani'],
                    ['rap', 'Rapanui'],
                    ['rar', 'Rarotongan; Cook Islands Maori'],
                    ['roa', 'Romance languages'],
                    ['roh', 'Romansh'],
                    ['rom', 'Romany'],
                    ['rum', 'Romanian'],
                    ['run', 'Rundi'],
                    ['rup', 'Aromanian; Arumanian; Macedo-Romanian'],
                    ['rus', 'Russian'],
                    ['sad', 'Sandawe'],
                    ['sag', 'Sango'],
                    ['sah', 'Yakut'],
                    ['sai', 'South American Indian languages'],
                    ['sal', 'Salishan languages'],
                    ['sam', 'Samaritan Aramaic'],
                    ['san', 'Sanskrit'],
                    ['sas', 'Sasak'],
                    ['sat', 'Santali'],
                    ['scn', 'Sicilian'],
                    ['sco', 'Scots'],
                    ['sel', 'Selkup'],
                    ['sem', 'Semitic languages'],
                    ['sga', 'Irish, Old (to 900)'],
                    ['sgn', 'Sign Languages'],
                    ['shn', 'Shan'],
                    ['sid', 'Sidamo'],
                    ['sin', 'Sinhala; Sinhalese'],
                    ['sio', 'Siouan languages'],
                    ['sit', 'Sino-Tibetan languages'],
                    ['sla', 'Slavic languages'],
                    ['slo', 'Slovak'],
                    ['slv', 'Slovenian'],
                    ['sma', 'Southern Sami'],
                    ['sme', 'Northern Sami'],
                    ['smi', 'Sami languages'],
                    ['smj', 'Lule Sami'],
                    ['smn', 'Inari Sami'],
                    ['smo', 'Samoan'],
                    ['sms', 'Skolt Sami'],
                    ['sna', 'Shona'],
                    ['snd', 'Sindhi'],
                    ['snk', 'Soninke'],
                    ['sog', 'Sogdian'],
                    ['som', 'Somali'],
                    ['son', 'Songhai languages'],
                    ['sot', 'Sotho, Southern'],
                    ['spa', 'Spanish; Castilian'],
                    ['srd', 'Sardinian'],
                    ['srn', 'Sranan Tongo'],
                    ['srp', 'Serbian'],
                    ['srr', 'Serer'],
                    ['ssa', 'Nilo-Saharan languages'],
                    ['ssw', 'Swati'],
                    ['suk', 'Sukuma'],
                    ['sun', 'Sundanese'],
                    ['sus', 'Susu'],
                    ['sux', 'Sumerian'],
                    ['swa', 'Swahili'],
                    ['swe', 'Swedish'],
                    ['syc', 'Classical Syriac'],
                    ['syr', 'Syriac'],
                    ['tah', 'Tahitian'],
                    ['tai', 'Tai languages'],
                    ['tam', 'Tamil'],
                    ['tat', 'Tatar'],
                    ['tel', 'Telugu'],
                    ['tem', 'Timne'],
                    ['ter', 'Tereno'],
                    ['tet', 'Tetum'],
                    ['tgk', 'Tajik'],
                    ['tgl', 'Tagalog'],
                    ['tha', 'Thai'],
                    ['tib', 'Tibetan'],
                    ['tig', 'Tigre'],
                    ['tir', 'Tigrinya'],
                    ['tiv', 'Tiv'],
                    ['tkl', 'Tokelau'],
                    ['tlh', 'Klingon; tlhIngan-Hol'],
                    ['tli', 'Tlingit'],
                    ['tmh', 'Tamashek'],
                    ['tog', 'Tonga (Nyasa)'],
                    ['ton', 'Tonga (Tonga Islands)'],
                    ['tpi', 'Tok Pisin'],
                    ['tsi', 'Tsimshian'],
                    ['tsn', 'Tswana'],
                    ['tso', 'Tsonga'],
                    ['tuk', 'Turkmen'],
                    ['tum', 'Tumbuka'],
                    ['tup', 'Tupi languages'],
                    ['tur', 'Turkish'],
                    ['tut', 'Altaic languages'],
                    ['tvl', 'Tuvalu'],
                    ['twi', 'Twi'],
                    ['tyv', 'Tuvinian'],
                    ['udm', 'Udmurt'],
                    ['uga', 'Ugaritic'],
                    ['uig', 'Uighur; Uyghur'],
                    ['ukr', 'Ukrainian'],
                    ['umb', 'Umbundu'],
                    ['und', 'Undetermined'],
                    ['urd', 'Urdu'],
                    ['uzb', 'Uzbek'],
                    ['vai', 'Vai'],
                    ['ven', 'Venda'],
                    ['vie', 'Vietnamese'],
                    ['vol', 'Volap\xfck'],
                    ['vot', 'Votic'],
                    ['wak', 'Wakashan languages'],
                    ['wal', 'Wolaitta; Wolaytta'],
                    ['war', 'Waray'],
                    ['was', 'Washo'],
                    ['wel', 'Welsh'],
                    ['wen', 'Sorbian languages'],
                    ['wln', 'Walloon'],
                    ['wol', 'Wolof'],
                    ['xal', 'Kalmyk; Oirat'],
                    ['xho', 'Xhosa'],
                    ['yao', 'Yao'],
                    ['yap', 'Yapese'],
                    ['yid', 'Yiddish'],
                    ['yor', 'Yoruba'],
                    ['ypk', 'Yupik languages'],
                    ['zap', 'Zapotec'],
                    ['zbl', 'Blissymbols; Blissymbolics; Bliss'],
                    ['zen', 'Zenaga'],
                    ['zgh', 'Standard Moroccan Tamazight'],
                    ['zha', 'Zhuang; Chuang'],
                    ['znd', 'Zande languages'],
                    ['zul', 'Zulu'],
                    ['zun', 'Zuni'],
                    ['zxx', 'No linguistic content; Not applicable'],
                    ['zza', 'Zaza; Dimili; Dimli; Kirdki; Kirmanjki; Zazaki']
                ], max_length=8, blank=True, null=True,
                verbose_name='Language'
            ),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.permissions import permission_acl_edit, permission_acl_view
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_facet, menu_list_facet, menu_main, menu_multi_item, menu_object,
    menu_secondary
)
from mayan.apps.events.classes import ModelEventType
from mayan.apps.events.links import (
    link_events_for_object, link_object_event_types_user_subcriptions_list,
)
from mayan.apps.events.permissions import permission_events_view
from mayan.apps.documents.search import document_page_search, document_search
from mayan.apps.navigation.classes import SourceColumn

from .events import (
    event_cabinet_edited, event_cabinet_add_document,
    event_cabinet_remove_document
)
from .links import (
    link_cabinet_list, link_document_cabinet_list,
    link_document_cabinet_remove, link_document_cabinet_add,
    link_document_multiple_cabinet_add, link_cabinet_child_add,
    link_cabinet_create, link_cabinet_delete, link_cabinet_edit,
    link_cabinet_view, link_custom_acl_list,
    link_multiple_document_cabinet_remove
)
from .menus import menu_cabinets
from .methods import method_get_document_cabinets
from .permissions import (
    permission_cabinet_add_document, permission_cabinet_delete,
    permission_cabinet_edit, permission_cabinet_remove_document,
    permission_cabinet_view
)
from .widgets import widget_document_cabinets


class CabinetsApp(MayanAppConfig):
    app_namespace = 'cabinets'
    app_url = 'cabinets'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.cabinets'
    verbose_name = _('Cabinets')

    def ready(self):
        super(CabinetsApp, self).ready()
        from actstream import registry
        from .wizard_steps import WizardStepCabinets  # NOQA

        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        DocumentCabinet = self.get_model(model_name='DocumentCabinet')
        Cabinet = self.get_model(model_name='Cabinet')

        # Add explicit order_by as DocumentCabinet ordering Meta option has no
        # effect.
        Document.add_to_class(
            name='document_cabinets', value=method_get_document_cabinets
        )

        ModelEventType.register(
            model=Cabinet, event_types=(
                event_cabinet_edited, event_cabinet_add_document,
                event_cabinet_remove_document
            )
        )

        ModelPermission.register(
            model=Document, permissions=(
                permission_cabinet_add_document,
                permission_cabinet_remove_document,
                permission_events_view
            )
        )

        ModelPermission.register(
            model=Cabinet, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_cabinet_delete, permission_cabinet_edit,
                permission_cabinet_view, permission_cabinet_add_document,
                permission_cabinet_remove_document
            )
        )

        def get_root_filter():
            return {
                'acl_filter': {'level': 0},
                'acl_values': ('tree_id',),
                'field_lookup': 'tree_id__in'
            }

        ModelPermission.register_function(
            model=Cabinet, function=get_root_filter
        )

        SourceColumn(
            attribute='label', is_identifier=True, is_sortable=True,
            source=Cabinet
        )

        SourceColumn(
            source=Document, label=_('Cabinets'),
            func=lambda context: widget_document_cabinets(
                document=context['object'], user=context['request'].user
            ), order=1
        )

        document_page_search.add_model_field(
            field='document_version__document__cabinets__label',
            label=_('Cabinets')
        )
        document_search.add_model_field(
            field='cabinets__label', label=_('Cabinets')
        )

        menu_facet.bind_links(
            links=(link_document_cabinet_list,), sources=(Document,)
        )

        menu_cabinets.bind_links(
            links=(
                link_cabinet_list, link_cabinet_create
            )
        )
        menu_list_facet.bind_links(
            links=(
                link_cabinet_view, link_custom_acl_list,
                link_events_for_object,
                link_object_event_types_user_subcriptions_list,
            ),
            sources=(Cabinet,)
        )

        menu_main.bind_links(links=(menu_cabinets,), position=98)

        menu_multi_item.bind_links(
            links=(
                link_document_multiple_cabinet_add,
                link_multiple_document_cabinet_remove
            ), sources=(Document,)
        )
        menu_object.bind_links(
            links=(
                link_cabinet_view,
            ), sources=(DocumentCabinet, )
        )
        menu_object.bind_links(
            links=(
                link_cabinet_delete, link_cabinet_edit, link_cabinet_child_add
            ), sources=(Cabinet,)
        )
        menu_object.unbind_links(
            links=(
                link_cabinet_delete, link_cabinet_edit, link_cabinet_child_add
            ), sources=(DocumentCabinet,)
        )
        menu_secondary.bind_links(
            links=(link_document_cabinet_add, link_document_cabinet_remove),
            sources=(
                'cabinets:document_cabinet_list',
                'cabinets:document_cabinet_add',
                'cabinets:document_cabinet_remove'
            )
        )

        registry.register(Cabinet)



from __future__ import unicode_literals

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.tests.mixins import DocumentTestMixin

from .mixins import TagTestMixin


class TagTestCase(DocumentTestMixin, TagTestMixin, BaseTestCase):
    auto_upload_document = False

    def test_document_addition(self):
        self._create_test_tag()
        self.upload_document()

        self.test_tag.documents.add(self.test_document)

        self.assertTrue(self.test_document in self.test_tag.documents.all())

    def test_document_remove(self):
        self._create_test_tag()
        self.upload_document()

        self.test_tag.documents.remove(self.test_document)

        self.assertTrue(self.test_document not in self.test_tag.documents.all())



from __future__ import absolute_import, unicode_literals

import hashlib
import logging
import os
import shutil
import uuid

from django.apps import apps
from django.db import models, transaction
from django.template import Template, Context
from django.urls import reverse
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from mayan.apps.converter.exceptions import InvalidOfficeFormat, PageCountError
from mayan.apps.converter.layers import layer_saved_transformations
from mayan.apps.converter.transformations import TransformationRotate
from mayan.apps.converter.utils import get_converter_class
from mayan.apps.mimetype.api import get_mimetype

from ..events import event_document_new_version, event_document_version_revert
from ..literals import DOCUMENT_IMAGES_CACHE_NAME
from ..managers import DocumentVersionManager
from ..settings import setting_fix_orientation, setting_hash_block_size
from ..signals import post_document_created, post_version_upload
from ..storages import storage_documentversion

from .document_models import Document

__all__ = ('DocumentVersion',)
logger = logging.getLogger(__name__)


# document image cache name hash function
def hash_function():
    return hashlib.sha256()


def UUID_FUNCTION(*args, **kwargs):
    return force_text(uuid.uuid4())


@python_2_unicode_compatible
class DocumentVersion(models.Model):
    """
    Model that describes a document version and its properties
    Fields:
    * mimetype - File mimetype. MIME types are a standard way to describe the
    format of a file, in this case the file format of the document.
    Some examples: "text/plain" or "image/jpeg". Mayan uses this to determine
    how to render a document's file. More information:
    http://www.freeformatter.com/mime-types-list.html
    * encoding - File Encoding. The filesystem encoding of the document's
    file: binary 7-bit, binary 8-bit, text, base64, etc.
    * checksum - A hash/checkdigit/fingerprint generated from the document's
    binary data. Only identical documents will have the same checksum. If a
    document is modified after upload it's checksum will not match, used for
    detecting file tampering among other things.
    """
    _pre_open_hooks = {}
    _post_save_hooks = {}

    document = models.ForeignKey(
        on_delete=models.CASCADE, related_name='versions', to=Document,
        verbose_name=_('Document')
    )
    timestamp = models.DateTimeField(
        auto_now_add=True, db_index=True, help_text=_(
            'The server date and time when the document version was processed.'
        ), verbose_name=_('Timestamp')
    )
    comment = models.TextField(
        blank=True, default='', help_text=_(
            'An optional short text describing the document version.'
        ), verbose_name=_('Comment')
    )

    # File related fields
    file = models.FileField(
        storage=storage_documentversion, upload_to=UUID_FUNCTION,
        verbose_name=_('File')
    )
    mimetype = models.CharField(
        blank=True, editable=False, help_text=_(
            'The document version\'s file mimetype. MIME types are a '
            'standard way to describe the format of a file, in this case '
            'the file format of the document. Some examples: "text/plain" '
            'or "image/jpeg". '
        ), max_length=255, null=True, verbose_name=_('MIME type')
    )
    encoding = models.CharField(
        blank=True, editable=False, help_text=_(
            'The document version file encoding. binary 7-bit, binary 8-bit, '
            'text, base64, etc.'
        ), max_length=64, null=True, verbose_name=_('Encoding')
    )
    checksum = models.CharField(
        blank=True, db_index=True, editable=False, help_text=(
            'A hash/checkdigit/fingerprint generated from the document\'s '
            'binary data. Only identical documents will have the same '
            'checksum.'
        ), max_length=64, null=True, verbose_name=_('Checksum')
    )

    class Meta:
        ordering = ('timestamp',)
        verbose_name = _('Document version')
        verbose_name_plural = _('Document version')

    objects = DocumentVersionManager()

    @classmethod
    def register_pre_open_hook(cls, order, func):
        cls._pre_open_hooks[order] = func

    @classmethod
    def register_post_save_hook(cls, order, func):
        cls._post_save_hooks[order] = func

    def __str__(self):
        return self.get_rendered_string()

    @cached_property
    def cache(self):
        Cache = apps.get_model(app_label='file_caching', model_name='Cache')
        return Cache.objects.get(name=DOCUMENT_IMAGES_CACHE_NAME)

    @cached_property
    def cache_partition(self):
        partition, created = self.cache.partitions.get_or_create(
            name='version-{}'.format(self.uuid)
        )
        return partition

    def delete(self, *args, **kwargs):
        for page in self.pages.all():
            page.delete()

        self.file.storage.delete(self.file.name)
        self.cache_partition.delete()

        return super(DocumentVersion, self).delete(*args, **kwargs)

    def exists(self):
        """
        Returns a boolean value that indicates if the document's file
        exists in storage. Returns True if the document's file is verified to
        be in the document storage. This is a diagnostic flag to help users
        detect if the storage has desynchronized (ie: Amazon's S3).
        """
        return self.file.storage.exists(self.file.name)

    def fix_orientation(self):
        for page in self.pages.all():
            degrees = page.detect_orientation()
            if degrees:
                layer_saved_transformations.add_transformation_to(
                    obj=page, transformation_class=TransformationRotate,
                    arguments='{{"degrees": {}}}'.format(360 - degrees)
                )

    def get_absolute_url(self):
        return reverse(
            viewname='documents:document_version_view', kwargs={
                'pk': self.pk
            }
        )

    def get_api_image_url(self, *args, **kwargs):
        first_page = self.pages.first()
        if first_page:
            return first_page.get_api_image_url(*args, **kwargs)

    def get_intermediate_file(self):
        cache_filename = 'intermediate_file'
        cache_file = self.cache_partition.get_file(filename=cache_filename)
        if cache_file:
            logger.debug('Intermidiate file found.')
            return cache_file.open()
        else:
            logger.debug('Intermidiate file not found.')

            try:
                with self.open() as version_file_object:
                    converter = get_converter_class()(
                        file_object=version_file_object
                    )
                    with converter.to_pdf() as pdf_file_object:
                        with self.cache_partition.create_file(filename=cache_filename) as file_object:
                            shutil.copyfileobj(
                                fsrc=pdf_file_object, fdst=file_object
                            )

                        return self.cache_partition.get_file(filename=cache_filename).open()
            except InvalidOfficeFormat:
                return self.open()
            except Exception as exception:
                logger.error(
                    'Error creating intermediate file "%s"; %s.',
                    cache_filename, exception
                )
                cache_file = self.cache_partition.get_file(filename=cache_filename)
                if cache_file:
                    cache_file.delete()
                raise

    def get_rendered_string(self, preserve_extension=False):
        if preserve_extension:
            filename, extension = os.path.splitext(self.document.label)
            return '{} ({}){}'.format(
                filename, self.get_rendered_timestamp(), extension
            )
        else:
            return Template(
                '{{ instance.document }} - {{ instance.timestamp }}'
            ).render(context=Context({'instance': self}))

    def get_rendered_timestamp(self):
        return Template('{{ instance.timestamp }}').render(
            context=Context({'instance': self})
        )

    def natural_key(self):
        return (self.checksum, self.document.natural_key())
    natural_key.dependencies = ['documents.Document']

    @property
    def is_in_trash(self):
        return self.document.is_in_trash

    def open(self, raw=False):
        """
        Return a file descriptor to a document version's file irrespective of
        the storage backend
        """
        if raw:
            return self.file.storage.open(self.file.name)
        else:
            result = self.file.storage.open(self.file.name)
            for key in sorted(DocumentVersion._pre_open_hooks):
                result = DocumentVersion._pre_open_hooks[key](
                    file_object=result, document_version=self
                )

            return result

    @property
    def pages_all(self):
        DocumentPage = apps.get_model(
            app_label='documents', model_name='DocumentPage'
        )
        return DocumentPage.passthrough.filter(document_version=self)

    @property
    def page_count(self):
        """
        The number of pages that the document posses.
        """
        return self.pages.count()

    @property
    def pages(self):
        return self.version_pages.all()

    def revert(self, _user=None):
        """
        Delete the subsequent versions after this one
        """
        logger.info(
            'Reverting to document document: %s to version: %s',
            self.document, self
        )

        with transaction.atomic():
            event_document_version_revert.commit(
                actor=_user, target=self.document
            )
            for version in self.document.versions.filter(timestamp__gt=self.timestamp):
                version.delete()

    def save(self, *args, **kwargs):
        """
        Overloaded save method that updates the document version's checksum,
        mimetype, and page count when created
        """
        user = kwargs.pop('_user', None)
        new_document_version = not self.pk

        if new_document_version:
            logger.info('Creating new version for document: %s', self.document)

        try:
            with transaction.atomic():
                super(DocumentVersion, self).save(*args, **kwargs)

                for key in sorted(DocumentVersion._post_save_hooks):
                    DocumentVersion._post_save_hooks[key](
                        document_version=self
                    )

                if new_document_version:
                    # Only do this for new documents
                    self.update_checksum(save=False)
                    self.update_mimetype(save=False)
                    self.save()
                    self.update_page_count(save=False)
                    if setting_fix_orientation.value:
                        self.fix_orientation()

                    logger.info(
                        'New document version "%s" created for document: %s',
                        self, self.document
                    )

                    self.document.is_stub = False
                    if not self.document.label:
                        self.document.label = force_text(self.file)

                    self.document.save(_commit_events=False)
        except Exception as exception:
            logger.error(
                'Error creating new document version for document "%s"; %s',
                self.document, exception
            )
            raise
        else:
            if new_document_version:
                event_document_new_version.commit(
                    actor=user, target=self, action_object=self.document
                )
                post_version_upload.send(sender=DocumentVersion, instance=self)

                if tuple(self.document.versions.all()) == (self,):
                    post_document_created.send(
                        sender=Document, instance=self.document
                    )

    def save_to_file(self, file_object):
        """
        Save a copy of the document from the document storage backend
        to the local filesystem
        """
        with self.open() as input_file_object:
            shutil.copyfileobj(fsrc=input_file_object, fdst=file_object)

    @property
    def size(self):
        if self.exists():
            return self.file.storage.size(self.file.name)
        else:
            return None

    def update_checksum(self, save=True):
        """
        Open a document version's file and update the checksum field using
        the user provided checksum function
        """
        block_size = setting_hash_block_size.value
        if block_size == 0:
            # If the setting value is 0 that means disable read limit. To disable
            # the read limit passing None won't work, we pass -1 instead as per
            # the Python documentation.
            # https://docs.python.org/2/tutorial/inputoutput.html#methods-of-file-objects
            block_size = -1

        if self.exists():
            hash_object = hash_function()
            with self.open() as file_object:
                while (True):
                    data = file_object.read(block_size)
                    if not data:
                        break

                    hash_object.update(data)

            self.checksum = force_text(hash_object.hexdigest())
            if save:
                self.save()

    def update_mimetype(self, save=True):
        """
        Read a document verions's file and determine the mimetype by calling
        the get_mimetype wrapper
        """
        if self.exists():
            try:
                with self.open() as file_object:
                    self.mimetype, self.encoding = get_mimetype(
                        file_object=file_object
                    )
            except Exception:
                self.mimetype = ''
                self.encoding = ''
            finally:
                if save:
                    self.save()

    def update_page_count(self, save=True):
        try:
            with self.open() as file_object:
                converter = get_converter_class()(
                    file_object=file_object, mime_type=self.mimetype
                )
                detected_pages = converter.get_page_count()
        except PageCountError:
            # If converter backend doesn't understand the format,
            # use 1 as the total page count
            pass
        else:
            DocumentPage = apps.get_model(
                app_label='documents', model_name='DocumentPage'
            )

            with transaction.atomic():
                self.pages.all().delete()

                for page_number in range(detected_pages):
                    DocumentPage.objects.create(
                        document_version=self, page_number=page_number + 1
                    )

            if save:
                self.save()

            return detected_pages

    @property
    def uuid(self):
        # Make cache UUID a mix of document UUID, version ID
        return '{}-{}'.format(self.document.uuid, self.pk)



from __future__ import absolute_import, unicode_literals

import platform

if platform.system() in ('FreeBSD', 'OpenBSD', 'Darwin'):
    DEFAULT_TESSERACT_BINARY_PATH = '/usr/local/bin/tesseract'
else:
    DEFAULT_TESSERACT_BINARY_PATH = '/usr/bin/tesseract'

DEFAULT_TESSERACT_TIMEOUT = 600  # 600 seconds, 10 minutes



from __future__ import unicode_literals

from django.db import models

from mptt.managers import TreeManager


class DocumentIndexInstanceNodeManager(models.Manager):
    def get_for(self, document):
        return self.filter(documents=document)


class IndexManager(models.Manager):
    def get_by_natural_key(self, slug):
        return self.get(slug=slug)

    def index_document(self, document):
        for index in self.filter(enabled=True, document_types=document.document_type):
            index.index_document(document=document)

    def rebuild(self):
        for index in self.all():
            index.rebuild()


class IndexInstanceNodeManager(TreeManager):
    def delete_empty(self):
        # Select leaf nodes only because .delete_empty() bubbles up
        for root_nodes in self.filter(parent=None):
            for index_instance_node in root_nodes.get_leafnodes():
                index_instance_node.delete_empty()

    def remove_document(self, document):
        for index_instance_node in self.filter(documents=document):
            index_instance_node.remove_document(document=document)



from __future__ import unicode_literals

from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils.deprecation import MiddlewareMixin


class AjaxRedirect(MiddlewareMixin):
    def process_request(self, request):
        ajax_referer = request.META.get('HTTP_X_ALT_REFERER')

        if ajax_referer:
            request.META['HTTP_REFERER'] = ajax_referer

        return None

    def process_response(self, request, response):
        if request.is_ajax():
            if type(response) == HttpResponseRedirect:
                response.status_code = getattr(
                    settings, 'AJAX_REDIRECT_CODE', 302
                )
        return response



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_slow

queue_ocr = CeleryQueue(name='parsing', label=_('Parsing'), worker=worker_slow)
queue_ocr.add_task_type(
    dotted_path='mayan.apps.document_parsing.tasks.task_parse_document_version',
    label=_('Document version parsing')
)



from __future__ import unicode_literals

from ..models import Index

from .literals import (
    TEST_INDEX_LABEL, TEST_INDEX_LABEL_EDITED, TEST_INDEX_SLUG,
    TEST_INDEX_TEMPLATE_DOCUMENT_LABEL_EXPRESSION
)


class IndexTestMixin(object):
    def _create_test_index(self, rebuild=False):
        # Create empty index
        self.test_index = Index.objects.create(label=TEST_INDEX_LABEL)

        # Add our document type to the new index
        if hasattr(self, 'test_document_type'):
            self.test_index.document_types.add(self.test_document_type)

        # Rebuild indexes
        if rebuild:
            Index.objects.rebuild()

    def _create_test_index_template_node(self):
        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_INDEX_TEMPLATE_DOCUMENT_LABEL_EXPRESSION,
            link_documents=True
        )


class IndexViewTestMixin(object):
    def _request_test_index_create_view(self):
        # Typecast to list to force queryset evaluation
        values = list(Index.objects.values_list('pk', flat=True))

        response = self.post(
            viewname='indexing:index_setup_create', data={
                'label': TEST_INDEX_LABEL, 'slug': TEST_INDEX_SLUG
            }
        )

        self.test_index = Index.objects.exclude(pk__in=values).first()

        return response

    def _request_test_index_delete_view(self):
        return self.post(
            viewname='indexing:index_setup_delete', kwargs={
                'pk': self.test_index.pk
            }
        )

    def _request_test_index_edit_view(self):
        return self.post(
            viewname='indexing:index_setup_edit', kwargs={
                'pk': self.test_index.pk
            }, data={
                'label': TEST_INDEX_LABEL_EDITED, 'slug': TEST_INDEX_SLUG
            }
        )

    def _request_test_index_rebuild_view(self):
        return self.post(
            viewname='indexing:index_setup_rebuild', kwargs={
                'pk': self.test_index.pk
            }
        )



from __future__ import unicode_literals

from actstream.models import Action

from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..events import (
    event_document_comment_created, event_document_comment_deleted,
    event_document_comment_edited
)
from ..models import Comment
from ..permissions import (
    permission_document_comment_create, permission_document_comment_delete,
    permission_document_comment_edit
)

from .mixins import DocumentCommentTestMixin, DocumentCommentViewTestMixin


class CommentEventsTestCase(
    DocumentCommentTestMixin, DocumentCommentViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_comment_create_event_no_permissions(self):
        action_count = Action.objects.count()

        response = self._request_test_comment_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Action.objects.count(), action_count)

    def test_comment_create_event_with_permissions(self):
        self.grant_permission(permission=permission_document_comment_create)

        action_count = Action.objects.count()

        response = self._request_test_comment_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Action.objects.count(), action_count + 1)

        event = Action.objects.first()

        comment = Comment.objects.first()

        self.assertEqual(event.action_object, self.test_document)
        self.assertEqual(event.actor, self._test_case_user)
        self.assertEqual(event.target, comment)
        self.assertEqual(event.verb, event_document_comment_created.id)

    def test_comment_delete_event_no_permissions(self):
        self._create_test_comment()

        action_count = Action.objects.count()

        response = self._request_test_comment_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Action.objects.count(), action_count)

    def test_comment_delete_event_with_access(self):
        self._create_test_comment()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_comment_delete
        )

        action_count = Action.objects.count()

        response = self._request_test_comment_delete_view()
        self.assertEqual(response.status_code, 302)
        # Total count remains the same. Document comment created is removed due
        # to cascade delete, document comment deleted event is added.
        self.assertEqual(Action.objects.count(), action_count)

        event = Action.objects.first()

        self.assertEqual(event.actor, self._test_case_user)
        self.assertEqual(event.target, self.test_document)
        self.assertEqual(event.verb, event_document_comment_deleted.id)

    def test_comment_edit_event_no_permissions(self):
        self._create_test_comment()

        action_count = Action.objects.count()

        response = self._request_test_comment_edit_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Action.objects.count(), action_count)

    def test_comment_edit_event_with_access(self):
        self._create_test_comment()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_comment_edit
        )

        action_count = Action.objects.count()

        response = self._request_test_comment_edit_view()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Action.objects.count(), action_count + 1)

        event = Action.objects.first()

        self.assertEqual(event.action_object, self.test_document)
        self.assertEqual(event.actor, self._test_case_user)
        self.assertEqual(event.target, self.test_document_comment)
        self.assertEqual(event.verb, event_document_comment_edited.id)



from __future__ import unicode_literals

import requests

from .http_adapters import TestClientAdapter


def request_method_factory(test_case):
    def request(method, url, **kwargs):
        with requests.sessions.Session() as session:
            session.mount(
                prefix=test_case.testserver_prefix,
                adapter=TestClientAdapter(test_case=test_case)
            )
            return session.request(method=method, url=url, **kwargs)

    return request



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('Web links'), name='web_links')

permission_web_link_create = namespace.add_permission(
    label=_('Create new web links'), name='web_link_create'
)
permission_web_link_delete = namespace.add_permission(
    label=_('Delete web links'), name='web_link_delete'
)
permission_web_link_edit = namespace.add_permission(
    label=_('Edit web links'), name='web_link_edit'
)
permission_web_link_view = namespace.add_permission(
    label=_('View existing web links'), name='web_link_view'
)
permission_web_link_instance_view = namespace.add_permission(
    label=_('View web link instances'), name='web_link_instance_view'
)



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_slow

queue_mailing = CeleryQueue(label=_('Mailing'), name='mailing', worker=worker_slow)
queue_mailing.add_task_type(
    label=_('Send document'),
    dotted_path='mayan.apps.mailer.tasks.task_send_document'
)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('linking', '0005_auto_20150729_2344'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='smartlink',
            options={
                'ordering': ('label',), 'verbose_name': 'Smart link',
                'verbose_name_plural': 'Smart links'
            },
        ),
        migrations.AlterField(
            model_name='smartlink',
            name='label',
            field=models.CharField(
                db_index=True, max_length=96, verbose_name='Label'
            ),
        ),
    ]



from __future__ import unicode_literals

from django.contrib import admin

from .models import Tag


@admin.register(Tag)
class TagAdmin(admin.ModelAdmin):
    filter_horizontal = ('documents',)
    list_display = ('label', 'color', 'get_preview_widget')



from __future__ import unicode_literals

from mayan.apps.checkouts.models import NewVersionBlock
from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.documents.models import Document
from mayan.apps.documents.permissions import permission_document_create
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.documents.tests.literals import (
    TEST_COMPRESSED_DOCUMENT_PATH, TEST_DOCUMENT_DESCRIPTION,
    TEST_SMALL_DOCUMENT_CHECKSUM, TEST_SMALL_DOCUMENT_PATH
)

from ..links import link_document_version_upload
from ..literals import SOURCE_CHOICE_WEB_FORM, SOURCE_UNCOMPRESS_CHOICE_Y
from ..models import WebFormSource
from ..permissions import (
    permission_sources_setup_create, permission_sources_setup_delete,
    permission_sources_setup_view, permission_staging_file_delete
)

from .literals import TEST_SOURCE_LABEL, TEST_SOURCE_UNCOMPRESS_N
from .mixins import (
    StagingFolderTestMixin, StagingFolderViewTestMixin, SourceTestMixin,
    SourceViewTestMixin
)


class DocumentUploadWizardViewTestMixin(object):
    def _request_upload_wizard_view(self, document_path=TEST_SMALL_DOCUMENT_PATH):
        with open(document_path, mode='rb') as file_object:
            return self.post(
                viewname='sources:document_upload_interactive', kwargs={
                    'source_id': self.test_source.pk
                }, data={
                    'source-file': file_object,
                    'document_type_id': self.test_document_type.pk,
                }
            )

    def _request_upload_interactive_view(self):
        return self.get(
            viewname='sources:document_upload_interactive', data={
                'document_type_id': self.test_document_type.pk,
            }
        )


class DocumentUploadWizardViewTestCase(
    SourceTestMixin, DocumentUploadWizardViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_document = False

    def test_upload_compressed_file(self):
        self.test_source.uncompress = SOURCE_UNCOMPRESS_CHOICE_Y
        self.test_source.save()

        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )

        response = self._request_upload_wizard_view(
            document_path=TEST_COMPRESSED_DOCUMENT_PATH
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Document.objects.count(), 2)
        self.assertTrue(
            'first document.pdf' in Document.objects.values_list(
                'label', flat=True
            )
        )
        self.assertTrue(
            'second document.pdf' in Document.objects.values_list(
                'label', flat=True
            )
        )

    def test_upload_wizard_without_permission(self):
        response = self._request_upload_wizard_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Document.objects.count(), 0)

    def test_upload_wizard_with_permission(self):
        self.grant_permission(permission=permission_document_create)

        response = self._request_upload_wizard_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Document.objects.count(), 1)
        self.assertEqual(
            Document.objects.first().checksum, TEST_SMALL_DOCUMENT_CHECKSUM
        )

    def test_upload_wizard_with_document_type_access(self):
        """
        Test uploading of documents by granting the document create
        permssion for the document type to the user
        """
        # Create an access control entry giving the role the document
        # create permission for the selected document type.
        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )

        with open(TEST_SMALL_DOCUMENT_PATH, mode='rb') as file_object:
            response = self.post(
                viewname='sources:document_upload_interactive', kwargs={
                    'source_id': self.test_source.pk
                }, data={
                    'source-file': file_object,
                    'document_type_id': self.test_document_type.pk,
                }
            )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Document.objects.count(), 1)

    def test_upload_interactive_view_no_permission(self):
        response = self._request_upload_interactive_view()
        self.assertEqual(response.status_code, 403)

    def test_upload_interactive_view_with_access(self):
        self.grant_access(
            permission=permission_document_create, obj=self.test_document_type
        )
        response = self._request_upload_interactive_view()
        self.assertContains(
            response=response, text=self.test_source.label, status_code=200
        )


class DocumentUploadIssueTestCase(GenericDocumentViewTestCase):
    auto_upload_document = False
    auto_login_superuser = True
    auto_login_user = False
    create_test_case_superuser = True
    create_test_case_user = False

    def test_issue_25(self):
        # Create new webform source
        self.post(
            viewname='sources:setup_source_create',
            kwargs={'source_type': SOURCE_CHOICE_WEB_FORM},
            data={'label': 'test', 'uncompress': 'n', 'enabled': True}
        )
        self.assertEqual(WebFormSource.objects.count(), 1)

        # Upload the test document
        with open(TEST_SMALL_DOCUMENT_PATH, mode='rb') as file_object:
            self.post(
                viewname='sources:document_upload_interactive', data={
                    'document-language': 'eng',
                    'source-file': file_object,
                    'document_type_id': self.test_document_type.pk
                }
            )
        self.assertEqual(Document.objects.count(), 1)

        document = Document.objects.first()
        # Test for issue 25 during creation
        # ** description fields was removed from upload from **
        self.assertEqual(document.description, '')

        # Reset description
        document.description = TEST_DOCUMENT_DESCRIPTION
        document.save()
        self.assertEqual(document.description, TEST_DOCUMENT_DESCRIPTION)

        # Test for issue 25 during editing
        self.post(
            viewname='documents:document_edit', kwargs={'pk': document.pk},
            data={
                'description': TEST_DOCUMENT_DESCRIPTION,
                'language': document.language, 'label': document.label
            }
        )
        # Fetch document again and test description
        document = Document.objects.first()
        self.assertEqual(document.description, TEST_DOCUMENT_DESCRIPTION)


class NewDocumentVersionViewTestCase(GenericDocumentViewTestCase):
    auto_login_superuser = True
    auto_login_user = False
    create_test_case_superuser = True
    create_test_case_user = False

    def test_new_version_block(self):
        """
        Gitlab issue #231
        User shown option to upload new version of a document even though it
        is blocked by checkout - v2.0.0b2

        Expected results:
            - Link to upload version view should not resolve
            - Upload version view should reject request
        """
        NewVersionBlock.objects.block(self.test_document)

        response = self.post(
            viewname='sources:document_version_upload', kwargs={
                'document_pk': self.test_document.pk
            }, follow=True
        )

        self.assertContains(
            response=response, text='blocked from uploading',
            status_code=200
        )

        response = self.get(
            viewname='documents:document_version_list', kwargs={
                'pk': self.test_document.pk
            }, follow=True
        )

        # Needed by the url view resolver
        response.context.current_app = None
        resolved_link = link_document_version_upload.resolve(
            context=response.context
        )

        self.assertEqual(resolved_link, None)


class SourcesViewTestCase(
    SourceTestMixin, SourceViewTestMixin, GenericViewTestCase
):
    auto_create_test_source = False

    def test_source_create_view_no_permission(self):
        response = self._request_setup_source_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(WebFormSource.objects.count(), 0)

    def test_source_create_view_with_permission(self):
        self.grant_permission(permission=permission_sources_setup_create)

        response = self._request_setup_source_create_view()
        self.assertEqual(response.status_code, 302)

        webform_source = WebFormSource.objects.first()
        self.assertEqual(webform_source.label, TEST_SOURCE_LABEL)
        self.assertEqual(webform_source.uncompress, TEST_SOURCE_UNCOMPRESS_N)

    def test_source_delete_view_with_permission(self):
        self._create_test_source()

        self.grant_permission(permission=permission_sources_setup_delete)

        response = self._request_setup_source_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WebFormSource.objects.count(), 0)

    def test_source_delete_view_no_permission(self):
        self._create_test_source()

        response = self._request_setup_source_delete_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(WebFormSource.objects.count(), 1)

    def test_source_list_view_no_permission(self):
        self._create_test_source()

        response = self._request_setup_source_list_view()
        self.assertEqual(response.status_code, 403)

    def test_source_list_view_with_permission(self):
        self._create_test_source()

        self.grant_permission(permission=permission_sources_setup_view)

        response = self._request_setup_source_list_view()
        self.assertContains(
            response=response, text=self.test_source.label, status_code=200
        )


class StagingFolderViewTestCase(
    StagingFolderTestMixin, StagingFolderViewTestMixin, GenericViewTestCase
):
    def setUp(self):
        super(StagingFolderViewTestCase, self).setUp()
        self._create_test_stating_folder()
        self._copy_test_document()

    def test_staging_file_delete_no_permission(self):
        staging_file_count = len(list(self.test_staging_folder.get_files()))
        staging_file = list(self.test_staging_folder.get_files())[0]

        response = self._request_staging_file_delete_view(
            staging_folder=self.test_staging_folder, staging_file=staging_file
        )
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            staging_file_count,
            len(list(self.test_staging_folder.get_files()))
        )

    def test_staging_file_delete_with_permission(self):
        self.grant_permission(permission=permission_staging_file_delete)

        staging_file_count = len(list(self.test_staging_folder.get_files()))
        staging_file = list(self.test_staging_folder.get_files())[0]

        response = self._request_staging_file_delete_view(
            staging_folder=self.test_staging_folder, staging_file=staging_file
        )
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(
            staging_file_count,
            len(list(self.test_staging_folder.get_files()))
        )



from __future__ import unicode_literals

from rest_framework import serializers


class EndpointSerializer(serializers.Serializer):
    label = serializers.CharField(read_only=True)
    url = serializers.URLField(read_only=True)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-12-13 00:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mailer', '0005_auto_20170718_0123'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='usermailer',
            options={'ordering': ('label',), 'verbose_name': 'Mailing profile', 'verbose_name_plural': 'Mailing profiles'},
        ),
        migrations.AlterField(
            model_name='usermailer',
            name='label',
            field=models.CharField(help_text='A short text describing the mailing profile.', max_length=128, unique=True, verbose_name='Label'),
        ),
    ]



from __future__ import unicode_literals

from django.core.files.storage import FileSystemStorage
from django.utils.encoding import force_bytes

from mayan.apps.storage.utils import fs_cleanup, mkdtemp

from ..models import Cache

from .literals import (
    TEST_CACHE_LABEL, TEST_CACHE_MAXIMUM_SIZE, TEST_CACHE_NAME,
    TEST_CACHE_PARTITION_FILE_FILENAME, TEST_CACHE_PARTITION_FILE_SIZE,
    TEST_CACHE_PARTITION_NAME, TEST_CACHE_STORAGE_INSTANCE_PATH
)

test_storage = None


class CacheTestMixin(object):
    def setUp(self):
        super(CacheTestMixin, self).setUp()
        global test_storage
        self.temporary_directory = mkdtemp()
        test_storage = FileSystemStorage(location=self.temporary_directory)

    def tearDown(self):
        fs_cleanup(filename=self.temporary_directory)
        super(CacheTestMixin, self).tearDown()

    def _create_test_cache(self):
        self.test_cache = Cache.objects.create(
            label=TEST_CACHE_LABEL,
            storage_instance_path=TEST_CACHE_STORAGE_INSTANCE_PATH,
            maximum_size=TEST_CACHE_MAXIMUM_SIZE,
            name=TEST_CACHE_NAME,
        )

    def _create_test_cache_partition(self):
        self.test_cache_partition = self.test_cache.partitions.create(
            name=TEST_CACHE_PARTITION_NAME
        )

    def _create_test_cache_partition_file(self):
        with self.test_cache_partition.create_file(filename=TEST_CACHE_PARTITION_FILE_FILENAME) as file_object:
            file_object.write(
                force_bytes(' ' * TEST_CACHE_PARTITION_FILE_SIZE)
            )

        self.test_cache_partition_file = self.test_cache_partition.files.get(
            filename=TEST_CACHE_PARTITION_FILE_FILENAME
        )


class CacheViewTestMixin(object):
    def _request_test_cache_list_view(self):
        return self.get(viewname='file_caching:cache_list')

    def _request_test_cache_purge_view(self):
        return self.post(
            viewname='file_caching:cache_purge', kwargs={
                'cache_id': self.test_cache.pk
            }
        )

    def _request_test_cache_multiple_purge_view(self):
        return self.post(
            viewname='file_caching:cache_multiple_purge', data={
                'id_list': self.test_cache.pk
            }
        )



from __future__ import unicode_literals

from contextlib import contextmanager
import logging

from django.core.files.base import ContentFile
from django.db import models, transaction
from django.db.models import Sum
from django.template.defaultfilters import filesizeformat
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from mayan.apps.lock_manager.exceptions import LockError
from mayan.apps.lock_manager.runtime import locking_backend

from .events import (
    event_cache_created, event_cache_edited, event_cache_purged
)

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Cache(models.Model):
    name = models.CharField(
        db_index=True, help_text=_('Internal name of the cache.'),
        max_length=128, unique=True, verbose_name=_('Name')
    )
    label = models.CharField(
        help_text=_('A short text describing the cache.'), max_length=128,
        verbose_name=_('Label')
    )
    maximum_size = models.PositiveIntegerField(
        help_text=_('Maximum size of the cache in bytes.'),
        verbose_name=_('Maximum size')
    )
    storage_instance_path = models.CharField(
        help_text=_(
            'Dotted path to the actual storage class used for the cache.'
        ), max_length=255, unique=True, verbose_name=_('Storage instance path')
    )

    class Meta:
        verbose_name = _('Cache')
        verbose_name_plural = _('Caches')

    def __str__(self):
        return self.label

    def get_files(self):
        return CachePartitionFile.objects.filter(partition__cache__id=self.pk)

    def get_maximum_size_display(self):
        return filesizeformat(bytes_=self.maximum_size)

    get_maximum_size_display.help_text = _(
        'Size at which the cache will start deleting old entries.'
    )
    get_maximum_size_display.short_description = _('Maximum size')

    def get_total_size(self):
        """
        Return the actual usage of the cache.
        """
        return self.get_files().aggregate(
            file_size__sum=Sum('file_size')
        )['file_size__sum'] or 0

    def get_total_size_display(self):
        return filesizeformat(bytes_=self.get_total_size())

    get_total_size_display.short_description = _('Total size')
    get_total_size_display.help_text = _('Current size of the cache.')

    def prune(self):
        """
        Deletes files until the total size of the cache is below the allowed
        maximum size of the cache.
        """
        while self.get_total_size() > self.maximum_size:
            self.get_files().earliest().delete()

    def purge(self, _user=None):
        """
        Deletes the entire cache.
        """
        for partition in self.partitions.all():
            partition.purge()

        event_cache_purged.commit(actor=_user, target=self)

    def save(self, *args, **kwargs):
        _user = kwargs.pop('_user', None)
        with transaction.atomic():
            is_new = not self.pk
            result = super(Cache, self).save(*args, **kwargs)
            if is_new:
                event_cache_created.commit(
                    actor=_user, target=self
                )
            else:
                event_cache_edited.commit(
                    actor=_user, target=self
                )

        self.prune()
        return result

    @cached_property
    def storage(self):
        return import_string(self.storage_instance_path)


class CachePartition(models.Model):
    cache = models.ForeignKey(
        on_delete=models.CASCADE, related_name='partitions',
        to=Cache, verbose_name=_('Cache')
    )
    name = models.CharField(
        max_length=128, verbose_name=_('Name')
    )

    class Meta:
        unique_together = ('cache', 'name')
        verbose_name = _('Cache partition')
        verbose_name_plural = _('Cache partitions')

    @staticmethod
    def get_combined_filename(parent, filename):
        return '{}-{}'.format(parent, filename)

    @contextmanager
    def create_file(self, filename):
        lock_id = 'cache_partition-create_file-{}-{}'.format(self.pk, filename)
        try:
            logger.debug('trying to acquire lock: %s', lock_id)
            lock = locking_backend.acquire_lock(lock_id)
            logger.debug('acquired lock: %s', lock_id)
            try:
                self.cache.prune()

                # Since open "wb+" doesn't create files force the creation of an
                # empty file.
                self.cache.storage.delete(
                    name=self.get_full_filename(filename=filename)
                )
                self.cache.storage.save(
                    name=self.get_full_filename(filename=filename),
                    content=ContentFile(content='')
                )

                try:
                    with transaction.atomic():
                        partition_file = self.files.create(filename=filename)
                        yield partition_file.open(mode='wb')
                        partition_file.update_size()
                except Exception as exception:
                    logger.error(
                        'Unexpected exception while trying to save new '
                        'cache file; %s', exception
                    )
                    self.cache.storage.delete(
                        name=self.get_full_filename(filename=filename)
                    )
                    raise
            finally:
                lock.release()
        except LockError:
            logger.debug('unable to obtain lock: %s' % lock_id)
            raise

    def delete(self, *args, **kwargs):
        self.purge()
        return super(CachePartition, self).delete(*args, **kwargs)

    def get_file(self, filename):
        try:
            return self.files.get(filename=filename)
        except self.files.model.DoesNotExist:
            return None

    def get_full_filename(self, filename):
        return CachePartition.get_combined_filename(
            parent=self.name, filename=filename
        )

    def purge(self):
        for parition_file in self.files.all():
            parition_file.delete()


class CachePartitionFile(models.Model):
    partition = models.ForeignKey(
        on_delete=models.CASCADE, related_name='files',
        to=CachePartition, verbose_name=_('Cache partition')
    )
    datetime = models.DateTimeField(
        auto_now_add=True, db_index=True, verbose_name=_('Date time')
    )
    filename = models.CharField(max_length=255, verbose_name=_('Filename'))
    file_size = models.PositiveIntegerField(
        default=0, verbose_name=_('File size')
    )

    class Meta:
        get_latest_by = 'datetime'
        unique_together = ('partition', 'filename')
        verbose_name = _('Cache partition file')
        verbose_name_plural = _('Cache partition files')

    def delete(self, *args, **kwargs):
        self.partition.cache.storage.delete(name=self.full_filename)
        return super(CachePartitionFile, self).delete(*args, **kwargs)

    def exists(self):
        return self.partition.cache.storage.exists(name=self.full_filename)

    @cached_property
    def full_filename(self):
        return CachePartition.get_combined_filename(
            parent=self.partition.name, filename=self.filename
        )

    def open(self, mode='rb'):
        # Open the file for reading. If the file is written to, the
        # .update_size() must be called.
        try:
            return self.partition.cache.storage.open(
                name=self.full_filename, mode=mode
            )
        except Exception as exception:
            logger.error(
                'Unexpected exception opening the cache file; %s', exception
            )
            raise

    def update_size(self):
        self.file_size = self.partition.cache.storage.size(
            name=self.full_filename
        )
        self.save()



from __future__ import absolute_import, unicode_literals

from contextlib import contextmanager
import locale
import logging

from PIL import Image
import pyocr
import pyocr.builders

from ..classes import OCRBackendBase
from ..exceptions import OCRError

logger = logging.getLogger(__name__)


@contextmanager
def c_locale():
    locale_current = locale.getlocale()
    locale.setlocale(locale.LC_ALL, 'C')
    yield
    locale.setlocale(locale.LC_ALL, locale_current)


class PyOCR(OCRBackendBase):
    def __init__(self, *args, **kwargs):
        super(PyOCR, self).__init__(*args, **kwargs)

        self.languages = ()

        tools = pyocr.get_available_tools()
        if len(tools) == 0:
            raise OCRError('No OCR tool found')

        self.tool = tools[0]

        # The tools are returned in the recommended order of usage
        for tool in tools:
            if tool.__name__ == 'pyocr.libtesseract':
                self.tool = tool

        logger.debug('Will use tool \'%s\'', self.tool.get_name())

        with c_locale():
            self.languages = self.tool.get_available_languages()

        logger.debug('Available languages: %s', ', '.join(self.languages))

    def execute(self, *args, **kwargs):
        """
        Execute the command line binary of tesseract
        """
        super(PyOCR, self).execute(*args, **kwargs)

        image = Image.open(self.converter.get_page())
        try:
            with c_locale():
                result = self.tool.image_to_string(
                    image,
                    lang=self.language,
                    builder=pyocr.builders.TextBuilder()
                )
        except Exception as exception:
            error_message = (
                'Exception calling pyocr with language option: {}; {}'
            ).format(self.language, exception)

            if self.language not in self.languages:
                error_message = (
                    '{}\nThe requested OCR language "{}" is not '
                    'available and needs to be installed.\n'
                ).format(
                    error_message, self.language
                )

            logger.error(error_message)
            raise OCRError(error_message)
        else:
            return result



from __future__ import unicode_literals

from rest_framework import status

from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..permissions import (
    permission_ocr_document, permission_ocr_content_view,
)

from .literals import TEST_DOCUMENT_CONTENT


class OCRAPITestCase(DocumentTestMixin, BaseAPITestCase):
    def _request_document_ocr_submit_view(self):
        return self.post(
            viewname='rest_api:document-ocr-submit-view',
            kwargs={'pk': self.test_document.pk}
        )

    def test_submit_document_no_access(self):
        response = self._request_document_ocr_submit_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertFalse(
            hasattr(self.test_document.pages.first(), 'ocr_content')
        )

    def test_submit_document_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_ocr_document
        )
        response = self._request_document_ocr_submit_view()
        self.assertEqual(response.status_code, status.HTTP_202_ACCEPTED)

        self.assertTrue(
            hasattr(self.test_document.pages.first(), 'ocr_content')
        )

    def _request_document_version_ocr_submit_view(self):
        return self.post(
            viewname='rest_api:document-version-ocr-submit-view', kwargs={
                'document_pk': self.test_document.pk,
                'version_pk': self.test_document.latest_version.pk
            }
        )

    def test_submit_document_version_no_access(self):
        response = self._request_document_version_ocr_submit_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertFalse(
            hasattr(self.test_document.pages.first(), 'ocr_content')
        )

    def test_submit_document_version_with_access(self):
        self.grant_access(
            permission=permission_ocr_document, obj=self.test_document
        )
        response = self._request_document_version_ocr_submit_view()
        self.assertEqual(response.status_code, status.HTTP_202_ACCEPTED)

        self.assertTrue(
            hasattr(self.test_document.pages.first(), 'ocr_content')
        )

    def _request_document_page_content_view(self):
        return self.get(
            viewname='rest_api:document-page-ocr-content-view', kwargs={
                'document_pk': self.test_document.pk,
                'version_pk': self.test_document.latest_version.pk,
                'page_pk': self.test_document.latest_version.pages.first().pk,
            }
        )

    def test_get_document_version_page_content_no_access(self):
        response = self._request_document_page_content_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_document_version_page_content_with_access(self):
        self.test_document.submit_for_ocr()
        self.grant_access(
            permission=permission_ocr_content_view, obj=self.test_document
        )
        response = self._request_document_page_content_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertTrue(
            TEST_DOCUMENT_CONTENT in response.data['content']
        )



from __future__ import unicode_literals

from .runtime import cache


def handler_document_cache_delete(sender, **kwargs):
    cache.clear_document(document=kwargs['instance'])


def handler_node_cache_delete(sender, **kwargs):
    cache.clear_node(node=kwargs['instance'])



from __future__ import absolute_import, unicode_literals

from django.contrib.auth import get_user_model


def get_user_queryset():
    return get_user_model().objects.filter(is_superuser=False, is_staff=False)



from __future__ import absolute_import, unicode_literals

import logging

from furl import furl

from django.contrib import messages
from django.urls import reverse
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _, ungettext
from django.views.generic import RedirectView

from mayan.apps.common.generics import (
    MultipleObjectConfirmActionView, SimpleView, SingleObjectListView
)
from mayan.apps.common.mixins import ExternalObjectMixin
from mayan.apps.common.settings import setting_home_view
from mayan.apps.common.utils import resolve
from mayan.apps.converter.literals import DEFAULT_ROTATION, DEFAULT_ZOOM_LEVEL

from ..forms import DocumentPageForm
from ..icons import icon_document_pages
from ..links.document_version_page_links import link_document_update_page_count
from ..models import Document, DocumentPage
from ..permissions import permission_document_edit, permission_document_view
from ..settings import (
    setting_rotation_step, setting_zoom_percent_step, setting_zoom_max_level,
    setting_zoom_min_level
)

__all__ = (
    'DocumentPageDisable', 'DocumentPageEnable', 'DocumentPageListView',
    'DocumentPageNavigationFirst', 'DocumentPageNavigationLast',
    'DocumentPageNavigationNext', 'DocumentPageNavigationPrevious',
    'DocumentPageView', 'DocumentPageViewResetView',
    'DocumentPageInteractiveTransformation', 'DocumentPageZoomInView',
    'DocumentPageZoomOutView', 'DocumentPageRotateLeftView',
    'DocumentPageRotateRightView'
)
logger = logging.getLogger(__name__)


class DocumentPageListView(ExternalObjectMixin, SingleObjectListView):
    external_object_class = Document
    external_object_permission = permission_document_view
    external_object_pk_url_kwarg = 'pk'

    def get_extra_context(self):
        return {
            'hide_object': True,
            'list_as_items': True,
            'no_results_icon': icon_document_pages,
            'no_results_main_link': link_document_update_page_count.resolve(
                request=self.request, resolved_object=self.external_object
            ),
            'no_results_text': _(
                'This could mean that the document is of a format that is '
                'not supported, that it is corrupted or that the upload '
                'process was interrupted. Use the document page recalculation '
                'action to attempt to introspect the page count again.'
            ),
            'no_results_title': _('No document pages available'),
            'object': self.external_object,
            'title': _('Pages for document: %s') % self.external_object,
        }

    def get_source_queryset(self):
        return self.external_object.pages_all


class DocumentPageNavigationBase(ExternalObjectMixin, RedirectView):
    external_object_class = DocumentPage
    external_object_permission = permission_document_view
    external_object_pk_url_kwarg = 'pk'

    def get_object(self):
        return self.external_object

    def get_redirect_url(self, *args, **kwargs):
        """
        Attempt to jump to the same kind of view but resolved to a new
        object of the same kind.
        """
        previous_url = self.request.META.get('HTTP_REFERER', None)

        if not previous_url:
            try:
                previous_url = self.get_object().get_absolute_url()
            except AttributeError:
                previous_url = reverse(viewname=setting_home_view.value)

        parsed_url = furl(url=previous_url)

        # Obtain the view name to be able to resolve it back with new keyword
        # arguments.
        resolver_match = resolve(path=force_text(parsed_url.path))

        new_kwargs = self.get_new_kwargs()

        if set(new_kwargs) == set(resolver_match.kwargs):
            # It is the same type of object, reuse the URL to stay in the
            # same kind of view but pointing to a new object
            url = reverse(
                viewname=resolver_match.view_name, kwargs=new_kwargs
            )
        else:
            url = parsed_url.path

        # Update just the path to retain the querystring in case there is
        # transformation data.
        parsed_url.path = url

        return parsed_url.tostr()


class DocumentPageNavigationFirst(DocumentPageNavigationBase):
    def get_new_kwargs(self):
        document_page = self.get_object()

        return {'pk': document_page.siblings.first().pk}


class DocumentPageNavigationLast(DocumentPageNavigationBase):
    def get_new_kwargs(self):
        document_page = self.get_object()

        return {'pk': document_page.siblings.last().pk}


class DocumentPageNavigationNext(DocumentPageNavigationBase):
    def get_new_kwargs(self):
        document_page = self.get_object()

        new_document_page = document_page.siblings.filter(
            page_number__gt=document_page.page_number
        ).first()
        if new_document_page:
            return {'pk': new_document_page.pk}
        else:
            messages.warning(
                message=_(
                    'There are no more pages in this document'
                ), request=self.request
            )
            return {'pk': document_page.pk}


class DocumentPageNavigationPrevious(DocumentPageNavigationBase):
    def get_new_kwargs(self):
        document_page = self.get_object()

        new_document_page = document_page.siblings.filter(
            page_number__lt=document_page.page_number
        ).last()
        if new_document_page:
            return {'pk': new_document_page.pk}
        else:
            messages.warning(
                message=_(
                    'You are already at the first page of this document'
                ), request=self.request
            )
            return {'pk': document_page.pk}


class DocumentPageView(ExternalObjectMixin, SimpleView):
    external_object_class = DocumentPage
    external_object_permission = permission_document_view
    external_object_pk_url_kwarg = 'pk'
    template_name = 'appearance/generic_form.html'

    def get_extra_context(self):
        zoom = int(self.request.GET.get('zoom', DEFAULT_ZOOM_LEVEL))
        rotation = int(self.request.GET.get('rotation', DEFAULT_ROTATION))

        document_page_form = DocumentPageForm(
            instance=self.get_object(), rotation=rotation, zoom=zoom
        )

        base_title = _('Image of: %s') % self.get_object()

        if zoom != DEFAULT_ZOOM_LEVEL:
            zoom_text = '({}%)'.format(zoom)
        else:
            zoom_text = ''

        return {
            'form': document_page_form,
            'hide_labels': True,
            'navigation_object_list': ('page',),
            'page': self.get_object(),
            'rotation': rotation,
            'title': ' '.join((base_title, zoom_text)),
            'read_only': True,
            'zoom': zoom,
        }

    def get_object(self):
        return self.external_object


class DocumentPageViewResetView(RedirectView):
    pattern_name = 'documents:document_page_view'


class DocumentPageInteractiveTransformation(ExternalObjectMixin, RedirectView):
    external_object_class = DocumentPage
    external_object_permission = permission_document_view
    external_object_pk_url_kwarg = 'pk'

    def get_object(self):
        return self.external_object

    def get_redirect_url(self, *args, **kwargs):
        query_dict = {
            'rotation': self.request.GET.get('rotation', DEFAULT_ROTATION),
            'zoom': self.request.GET.get('zoom', DEFAULT_ZOOM_LEVEL)
        }

        url = furl(
            args=query_dict, path=reverse(
                viewname='documents:document_page_view',
                kwargs={'pk': self.kwargs['pk']}
            )

        )

        self.transformation_function(query_dict=query_dict)
        # Refresh query_dict to args reference
        url.args = query_dict

        return url.tostr()


class DocumentPageZoomInView(DocumentPageInteractiveTransformation):
    def transformation_function(self, query_dict):
        zoom = int(query_dict['zoom']) + setting_zoom_percent_step.value

        if zoom > setting_zoom_max_level.value:
            zoom = setting_zoom_max_level.value

        query_dict['zoom'] = zoom


class DocumentPageZoomOutView(DocumentPageInteractiveTransformation):
    def transformation_function(self, query_dict):
        zoom = int(query_dict['zoom']) - setting_zoom_percent_step.value

        if zoom < setting_zoom_min_level.value:
            zoom = setting_zoom_min_level.value

        query_dict['zoom'] = zoom


class DocumentPageRotateLeftView(DocumentPageInteractiveTransformation):
    def transformation_function(self, query_dict):
        query_dict['rotation'] = (
            int(query_dict['rotation']) - setting_rotation_step.value
        ) % 360


class DocumentPageRotateRightView(DocumentPageInteractiveTransformation):
    def transformation_function(self, query_dict):
        query_dict['rotation'] = (
            int(query_dict['rotation']) + setting_rotation_step.value
        ) % 360


class DocumentPageDisable(MultipleObjectConfirmActionView):
    object_permission = permission_document_edit
    pk_url_kwarg = 'pk'
    success_message_singular = '%(count)d document page disabled.'
    success_message_plural = '%(count)d document pages disabled.'

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Disable the selected document page?',
                plural='Disable the selected document pages?',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result['object'] = queryset.first()

        return result

    def get_source_queryset(self):
        return DocumentPage.passthrough.all()

    def object_action(self, form, instance):
        instance.enabled = False
        instance.save()


class DocumentPageEnable(MultipleObjectConfirmActionView):
    object_permission = permission_document_edit
    pk_url_kwarg = 'pk'
    success_message_singular = '%(count)d document page enabled.'
    success_message_plural = '%(count)d document pages enabled.'

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Enable the selected document page?',
                plural='Enable the selected document pages?',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result['object'] = queryset.first()

        return result

    def get_source_queryset(self):
        return DocumentPage.passthrough.all()

    def object_action(self, form, instance):
        instance.enabled = True
        instance.save()



from __future__ import unicode_literals

import os

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

from .setting_migrations import DocumentSignaturesSettingMigration

namespace = Namespace(
    label=_('Document signatures'),
    migration_class=DocumentSignaturesSettingMigration, name='signatures',
    version='0002'
)

setting_storage_backend = namespace.add_setting(
    default='django.core.files.storage.FileSystemStorage',
    global_name='SIGNATURES_STORAGE_BACKEND', help_text=_(
        'Path to the Storage subclass to use when storing detached '
        'signatures.'
    )
)
setting_storage_backend_arguments = namespace.add_setting(
    global_name='SIGNATURES_STORAGE_BACKEND_ARGUMENTS',
    default={
        'location': os.path.join(settings.MEDIA_ROOT, 'document_signatures')
    }, help_text=_(
        'Arguments to pass to the SIGNATURE_STORAGE_BACKEND. '
    )
)



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(
    name='document_states', label=_('Document workflows')
)

permission_workflow_create = namespace.add_permission(
    name='workflow_create', label=_('Create workflows')
)
permission_workflow_delete = namespace.add_permission(
    name='workflow_delte', label=_('Delete workflows')
)
permission_workflow_edit = namespace.add_permission(
    name='workflow_edit', label=_('Edit workflows')
)
permission_workflow_view = namespace.add_permission(
    name='workflow_view', label=_('View workflows')
)
# Translators: This text refers to the permission to grant user the ability to
# 'transition workflows' from one state to another, to move the workflow
# forwards
permission_workflow_transition = namespace.add_permission(
    name='workflow_transition', label=_('Transition workflows')
)
permission_workflow_tools = namespace.add_permission(
    name='workflow_tools', label=_('Execute workflow tools')
)



from __future__ import unicode_literals

import logging
import types

from django.conf import settings
from django.core.exceptions import FieldDoesNotExist
from django.db.models.constants import LOOKUP_SEP
from django.urls import resolve as django_resolve
from django.urls.base import get_script_prefix
from django.utils.encoding import force_text
from django.utils.six.moves import reduce as reduce_function

from mayan.apps.common.compat import dict_type, dictionary_type

from .literals import DJANGO_SQLITE_BACKEND

logger = logging.getLogger(__name__)


def check_for_sqlite():
    return settings.DATABASES['default']['ENGINE'] == DJANGO_SQLITE_BACKEND and settings.DEBUG is False


def convert_to_id_list(items):
    return ','.join(map(force_text, items))


def get_related_field(model, related_field_name):
    try:
        local_field_name, remaining_field_path = related_field_name.split(
            LOOKUP_SEP, 1
        )
    except ValueError:
        local_field_name = related_field_name
        remaining_field_path = None

    related_field = model._meta.get_field(local_field_name)

    if remaining_field_path:
        return get_related_field(
            model=related_field.related_model,
            related_field_name=remaining_field_path
        )

    return related_field


def introspect_attribute(attribute_name, obj):
    """
    Resolve the attribute of model. Supports nested reference using dotted
    paths or double underscore.
    """
    try:
        # Try as a related field
        obj._meta.get_field(field_name=attribute_name)
    except (AttributeError, FieldDoesNotExist):
        attribute_name = attribute_name.replace('__', '.')

        try:
            # If there are separators in the attribute name, traverse them
            # to the final attribute
            attribute_part, attribute_remaining = attribute_name.split(
                '.', 1
            )
        except ValueError:
            return attribute_name, obj
        else:
            related_field = obj._meta.get_field(field_name=attribute_part)
            return introspect_attribute(
                attribute_name=attribute_part,
                obj=related_field.related_model,
            )
    else:
        return attribute_name, obj


def resolve(path, urlconf=None):
    path = '/{}'.format(path.replace(get_script_prefix(), '', 1))
    return django_resolve(path=path, urlconf=urlconf)


def resolve_attribute(attribute, obj, kwargs=None):
    """
    Resolve the attribute of an object. Behaves like the Python REPL but with
    an unified dotted path schema regardless of the attribute type.
    Supports callables, dictionaries, properties, related model fields.
    """
    if not kwargs:
        kwargs = {}

    # Try as a callable
    try:
        return attribute(obj, **kwargs)
    except TypeError:
        # Try as a dictionary
        try:
            return obj[attribute]
        except TypeError:
            try:
                # If there are dots in the attribute name, traverse them
                # to the final attribute
                result = reduce_function(getattr, attribute.split('.'), obj)
                try:
                    # Try it as a method
                    return result(**kwargs)
                except TypeError:
                    # Try it as a property
                    return result
            except AttributeError:
                # Try as a related model field
                if LOOKUP_SEP in attribute:
                    attribute_replaced = attribute.replace(LOOKUP_SEP, '.')
                    return resolve_attribute(
                        obj=obj, attribute=attribute_replaced, kwargs=kwargs
                    )
                else:
                    raise


def return_attrib(obj, attrib, arguments=None):
    if isinstance(attrib, types.FunctionType):
        return attrib(obj)
    elif isinstance(
        obj, dict_type
    ) or isinstance(obj, dictionary_type):
        return obj[attrib]
    else:
        result = reduce_function(getattr, attrib.split('.'), obj)
        if isinstance(result, types.MethodType):
            if arguments:
                return result(**arguments)
            else:
                return result()
        else:
            return result


def return_related(instance, related_field):
    """
    This functions works in a similar method to return_attrib but is
    meant for related models. Support multiple levels of relationship
    using double underscore.
    """
    return reduce_function(getattr, related_field.split('__'), instance)



from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0026_auto_20150729_2140'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                ('comment', models.TextField(verbose_name='comment')),
                (
                    'submit_date', models.DateTimeField(
                        verbose_name='Date time submitted', db_index=True
                    )
                ),
                (
                    'document', models.ForeignKey(
                        related_name='comments', verbose_name='Document',
                        to='documents.Document'
                    )
                ),
                (
                    'user', models.ForeignKey(
                        related_name='comments', editable=False,
                        to=settings.AUTH_USER_MODEL, verbose_name='User'
                    )
                ),
            ],
            options={
                'ordering': ('-submit_date',),
                'get_latest_by': 'submit_date',
                'verbose_name': 'Comment',
                'verbose_name_plural': 'Comments',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.forms import FilteredSelectionForm

from .widgets import TagFormWidget


class TagMultipleSelectionForm(FilteredSelectionForm):
    class Media:
        js = ('tags/js/tags_form.js',)

    class Meta:
        allow_multiple = True
        field_name = 'tags'
        label = _('Tags')
        required = False
        widget_class = TagFormWidget
        widget_attributes = {'class': 'select2-tags'}



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APIStagingSourceFileView, APIStagingSourceFileImageView,
    APIStagingSourceListView, APIStagingSourceView
)
from .views import (
    SetupSourceCheckView, SetupSourceCreateView, SetupSourceDeleteView,
    SetupSourceEditView, SetupSourceListView, SourceLogListView,
    StagingFileDeleteView, DocumentVersionUploadInteractiveView, UploadInteractiveView
)
from .wizards import DocumentCreateWizard

urlpatterns = [
    url(
        regex=r'^staging_files/(?P<pk>\d+)/(?P<encoded_filename>.+)/delete/$',
        view=StagingFileDeleteView.as_view(), name='staging_file_delete'
    ),

    # Document create views

    url(
        regex=r'^documents/create/from/local/multiple/$',
        view=DocumentCreateWizard.as_view(), name='document_create_multiple'
    ),
    url(
        regex=r'^documents/upload/new/interactive/(?P<source_id>\d+)/$',
        view=UploadInteractiveView.as_view(),
        name='document_upload_interactive'
    ),
    url(
        regex=r'^documents/upload/new/interactive/$',
        view=UploadInteractiveView.as_view(),
        name='document_upload_interactive'
    ),
    url(
        regex=r'^documents/(?P<document_pk>\d+)/versions/upload/interactive/(?P<source_id>\d+)/$',
        view=DocumentVersionUploadInteractiveView.as_view(),
        name='document_version_upload'
    ),
    url(
        regex=r'^documents/(?P<document_pk>\d+)/versions/upload/interactive/$',
        view=DocumentVersionUploadInteractiveView.as_view(),
        name='document_version_upload'
    ),

    # Setup views

    url(
        regex=r'^sources/$', view=SetupSourceListView.as_view(),
        name='setup_source_list'
    ),
    url(
        regex=r'^sources/(?P<pk>\d+)/edit/$', view=SetupSourceEditView.as_view(),
        name='setup_source_edit'
    ),
    url(
        regex=r'^sources/(?P<pk>\d+)/logs/$', view=SourceLogListView.as_view(),
        name='setup_source_logs'
    ),
    url(
        regex=r'^sources/(?P<pk>\d+)/delete/$',
        view=SetupSourceDeleteView.as_view(), name='setup_source_delete'
    ),
    url(
        regex=r'^sources/(?P<source_type>\w+)/create/$',
        view=SetupSourceCreateView.as_view(), name='setup_source_create'
    ),
    url(
        regex=r'^sources/(?P<pk>\d+)/check/$',
        view=SetupSourceCheckView.as_view(), name='setup_source_check'
    ),
]

api_urls = [
    url(
        regex=r'^staging_folders/file/(?P<staging_folder_pk>[0-9]+)/(?P<encoded_filename>.+)/image/$',
        view=APIStagingSourceFileImageView.as_view(),
        name='stagingfolderfile-image-view'
    ),
    url(
        regex=r'^staging_folders/file/(?P<staging_folder_pk>[0-9]+)/(?P<encoded_filename>.+)/$',
        view=APIStagingSourceFileView.as_view(), name='stagingfolderfile-detail'
    ),
    url(
        regex=r'^staging_folders/$', view=APIStagingSourceListView.as_view(),
        name='stagingfolder-list'
    ),
    url(
        regex=r'^staging_folders/(?P<pk>[0-9]+)/$',
        view=APIStagingSourceView.as_view(), name='stagingfolder-detail'
    )
]



from __future__ import unicode_literals

from dateutil.parser import parse

from django.core.exceptions import ValidationError


class MetadataParser(object):
    _registry = []

    @classmethod
    def register(cls, parser):
        cls._registry.append(parser)

    @classmethod
    def get_all(cls):
        return cls._registry

    @classmethod
    def get_import_path(cls):
        return cls.__module__ + '.' + cls.__name__

    @classmethod
    def get_import_paths(cls):
        return [validator.get_import_path() for validator in cls.get_all()]

    def execute(self, input_data):
        raise NotImplementedError

    def parse(self, input_data):
        try:
            return self.execute(input_data)
        except Exception as exception:
            raise ValidationError(exception)


class DateAndTimeParser(MetadataParser):
    def execute(self, input_data):
        return parse(input_data).isoformat()


class DateParser(MetadataParser):
    def execute(self, input_data):
        return parse(input_data).date().isoformat()


class TimeParser(MetadataParser):
    def execute(self, input_data):
        return parse(input_data).time().isoformat()


MetadataParser.register(DateAndTimeParser)
MetadataParser.register(DateParser)
MetadataParser.register(TimeParser)



from __future__ import unicode_literals

from django.contrib import admin

from .models import LogEntry, UserMailer


@admin.register(LogEntry)
class LogEntryAdmin(admin.ModelAdmin):
    date_hierarchy = 'datetime'
    list_display = ('datetime', 'message')
    readonly_fields = ('datetime', 'message')


@admin.register(UserMailer)
class UserMailerAdmin(admin.ModelAdmin):
    list_display = (
        'label', 'default', 'enabled', 'backend_path', 'backend_data'
    )



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('metadata', '0003_auto_20150708_0323'),
    ]

    operations = [
        migrations.AlterField(
            model_name='metadatatype',
            name='label',
            field=models.CharField(max_length=48, verbose_name='Label'),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from .literals import (
    DEFAULT_DOCUMENT_TYPE_LABEL, DOCUMENT_CACHE_STORAGE_INSTANCE_PATH,
    DOCUMENT_IMAGES_CACHE_NAME
)
from .settings import setting_document_cache_maximum_size
from .signals import post_initial_document_type
from .tasks import task_clean_empty_duplicate_lists, task_scan_duplicates_for


def handler_create_default_document_type(sender, **kwargs):
    DocumentType = apps.get_model(
        app_label='documents', model_name='DocumentType'
    )

    if not DocumentType.objects.count():
        document_type = DocumentType.objects.create(
            label=DEFAULT_DOCUMENT_TYPE_LABEL
        )
        post_initial_document_type.send(
            sender=DocumentType, instance=document_type
        )


def handler_create_document_cache(sender, **kwargs):
    Cache = apps.get_model(app_label='file_caching', model_name='Cache')
    Cache.objects.update_or_create(
        defaults={
            'label': _('Document images'),
            'storage_instance_path': DOCUMENT_CACHE_STORAGE_INSTANCE_PATH,
            'maximum_size': setting_document_cache_maximum_size.value,
        }, name=DOCUMENT_IMAGES_CACHE_NAME,
    )


def handler_scan_duplicates_for(sender, instance, **kwargs):
    task_scan_duplicates_for.apply_async(
        kwargs={'document_id': instance.document.pk}
    )


def handler_remove_empty_duplicates_lists(sender, **kwargs):
    task_clean_empty_duplicate_lists.apply_async()



from __future__ import unicode_literals

from mayan.apps.converter.utils import get_converter_class


class OCRBackendBase(object):
    def execute(self, file_object, language=None, transformations=None):
        self.language = language

        if not transformations:
            transformations = []

        self.converter = get_converter_class()(file_object=file_object)

        for transformation in transformations:
            self.converter.transform(transformation=transformation)



from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType

from rest_framework import serializers


class ContentTypeSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('app_label', 'id', 'model')
        model = ContentType


class TemplateSerializer(serializers.Serializer):
    hex_hash = serializers.CharField(read_only=True)
    name = serializers.CharField(read_only=True)
    html = serializers.CharField(read_only=True)
    url = serializers.HyperlinkedIdentityField(
        lookup_field='name', lookup_url_kwarg='name',
        view_name='rest_api:template-detail'
    )



from __future__ import absolute_import, unicode_literals

import logging

from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.document_states.classes import WorkflowAction

from .models import Cabinet
from .permissions import (
    permission_cabinet_add_document, permission_cabinet_remove_document
)

__all__ = ('CabinetAddAction', 'CabinetRemoveAction')
logger = logging.getLogger(__name__)


class CabinetAddAction(WorkflowAction):
    fields = {
        'cabinets': {
            'label': _('Cabinets'),
            'class': 'django.forms.ModelMultipleChoiceField', 'kwargs': {
                'help_text': _(
                    'Cabinets to which the document will be added.'
                ),
                'queryset': Cabinet.objects.none(), 'required': False
            }
        },
    }
    label = _('Add to cabinets')
    widgets = {
        'cabinets': {
            'class': 'django.forms.widgets.SelectMultiple', 'kwargs': {
                'attrs': {'class': 'select2'},
            }
        }
    }
    permission = permission_cabinet_add_document

    def execute(self, context):
        for cabinet in self.get_cabinets():
            cabinet.document_add(document=context['document'])

    def get_form_schema(self, request):
        user = request.user
        logger.debug('user: %s', user)

        queryset = AccessControlList.objects.restrict_queryset(
            permission=self.permission, queryset=Cabinet.objects.all(),
            user=user
        )

        self.fields['cabinets']['kwargs']['queryset'] = queryset

        return {
            'fields': self.fields,
            'widgets': self.widgets
        }

    def get_cabinets(self):
        return Cabinet.objects.filter(pk__in=self.form_data.get('cabinets', ()))


class CabinetRemoveAction(CabinetAddAction):
    fields = {
        'cabinets': {
            'label': _('Cabinet'),
            'class': 'django.forms.ModelMultipleChoiceField', 'kwargs': {
                'help_text': _(
                    'Cabinets from which the document will be removed.'),
                'queryset': Cabinet.objects.none(), 'required': False
            }
        },
    }
    label = _('Remove from cabinets')
    permission = permission_cabinet_remove_document

    def execute(self, context):
        for cabinet in self.get_cabinets():
            cabinet.document_remove(document=context['document'])



from __future__ import print_function, unicode_literals

import logging

from fuse import FUSE

from django.core import management
from django.core.management.base import CommandError

from ...filesystems import IndexFilesystem

logger = logging.getLogger(__name__)


class Command(management.BaseCommand):
    help = 'Mount an index as a FUSE filesystem.'

    def add_arguments(self, parser):
        parser.add_argument('slug', nargs='?', help='Index slug')
        parser.add_argument('mount_point', nargs='?', help='Mount point')
        parser.add_argument(
            '--allow-other', action='store_true', dest='allow_other',
            default=False,
            help='All users (including root) can access the index files.'
        )
        parser.add_argument(
            '--allow-root', action='store_true', dest='allow_root',
            default=False,
            help='Mount access is limited to the user mounting the index and '
            'root. This option and --allow-other are mutually exclusive.'
        )

    def handle(self, *args, **options):
        if not options.get('slug') or not options.get('mount_point'):
            self.stderr.write(self.style.ERROR('Incorrect number of arguments'))
            exit(1)

        try:
            FUSE(
                operations=IndexFilesystem(index_slug=options['slug']),
                mountpoint=options['mount_point'], nothreads=True, foreground=True,
                allow_other=options['allow_other'],
                allow_root=options['allow_root']
            )
        except RuntimeError:
            if options['allow_other'] or options['allow_root']:
                raise CommandError(
                    'Make sure \'user_allow_other\' is set in /etc/fuse.conf'
                )
            else:
                raise



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('converter', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='transformation',
            old_name='transformation',
            new_name='name',
        ),
    ]



from __future__ import unicode_literals

from django import forms
from django.core.exceptions import ValidationError
from django.forms.formsets import formset_factory
from django.utils.translation import string_concat, ugettext_lazy as _

from .classes import MetadataLookup
from .models import DocumentTypeMetadataType, MetadataType


class DocumentMetadataForm(forms.Form):
    id = forms.CharField(label=_('ID'), widget=forms.HiddenInput)

    name = forms.CharField(
        label=_('Name'), required=False,
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )
    value = forms.CharField(
        label=_('Value'), required=False, widget=forms.TextInput(
            attrs={'class': 'metadata-value'}
        )
    )
    update = forms.BooleanField(
        initial=True, label=_('Update'), required=False
    )

    class Media:
        js = ('metadata/js/metadata_form.js',)

    def __init__(self, *args, **kwargs):
        super(DocumentMetadataForm, self).__init__(*args, **kwargs)

        # Set form fields initial values
        if 'initial' in kwargs:
            self.metadata_type = kwargs['initial']['metadata_type']
            self.document_type = kwargs['initial']['document_type']
            required_string = ''

            required = self.metadata_type.get_required_for(
                document_type=self.document_type
            )

            if required:
                self.fields['value'].required = True
                required_string = ' (%s)' % _('Required')
            else:
                self.fields['value'].required = False
                self.fields['update'].initial = False

            self.fields['name'].initial = '%s%s' % (
                (
                    self.metadata_type.label if self.metadata_type.label else self.metadata_type.name
                ),
                required_string
            )
            self.fields['id'].initial = self.metadata_type.pk

            if self.metadata_type.lookup:
                try:
                    self.fields['value'] = forms.ChoiceField(
                        label=self.fields['value'].label
                    )
                    choices = self.metadata_type.get_lookup_values()
                    choices = list(zip(choices, choices))
                    if not required:
                        choices.insert(0, ('', '------'))
                    self.fields['value'].choices = choices
                    self.fields['value'].required = required
                    self.fields['value'].widget.attrs.update(
                        {'class': 'metadata-value'}
                    )
                except Exception as exception:
                    self.fields['value'].initial = _(
                        'Lookup value error: %s'
                    ) % exception
                    self.fields['value'].widget = forms.TextInput(
                        attrs={'readonly': 'readonly'}
                    )

            if self.metadata_type.default:
                try:
                    self.fields[
                        'value'
                    ].initial = self.metadata_type.get_default_value()
                except Exception as exception:
                    self.fields['value'].initial = _(
                        'Default value error: %s'
                    ) % exception
                    self.fields['value'].widget = forms.TextInput(
                        attrs={'readonly': 'readonly'}
                    )

    def clean(self):
        metadata_type = getattr(self, 'metadata_type', None)

        if metadata_type:
            required = self.metadata_type.get_required_for(
                document_type=self.document_type
            )
            if required and not self.cleaned_data.get('update'):
                raise ValidationError(
                    _(
                        '"%s" is required for this document type.'
                    ) % self.metadata_type.label
                )

        if self.cleaned_data.get('update') and hasattr(self, 'metadata_type'):
            self.cleaned_data['value'] = self.metadata_type.validate_value(
                document_type=self.document_type,
                value=self.cleaned_data.get('value')
            )

        return self.cleaned_data


DocumentMetadataFormSet = formset_factory(DocumentMetadataForm, extra=0)


class DocumentMetadataAddForm(forms.Form):
    metadata_type = forms.ModelMultipleChoiceField(
        help_text=_('Metadata types to be added to the selected documents.'),
        label=_('Metadata type'), queryset=MetadataType.objects.all(),
        widget=forms.SelectMultiple(
            attrs={'class': 'select2'},
        )
    )

    def __init__(self, *args, **kwargs):
        document_type = kwargs.pop('document_type', None)

        if document_type:
            queryset = kwargs.pop(
                'queryset', MetadataType.objects.get_for_document_type(
                    document_type=document_type
                )
            )
        else:
            queryset = MetadataType.objects.none()

        super(DocumentMetadataAddForm, self).__init__(*args, **kwargs)

        self.fields['metadata_type'].queryset = queryset


class DocumentMetadataRemoveForm(DocumentMetadataForm):
    update = forms.BooleanField(
        initial=False, label=_('Remove'), required=False
    )

    def __init__(self, *args, **kwargs):
        super(DocumentMetadataRemoveForm, self).__init__(*args, **kwargs)
        self.fields.pop('value')

    def clean(self):
        return super(forms.Form, self).clean()


DocumentMetadataRemoveFormSet = formset_factory(
    DocumentMetadataRemoveForm, extra=0
)


class MetadataTypeForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(MetadataTypeForm, self).__init__(*args, **kwargs)
        self.fields['lookup'].help_text = string_concat(
            self.fields['lookup'].help_text,
            _(' Available template context variables: '),
            MetadataLookup.get_as_help_text()
        )

    class Meta:
        fields = ('name', 'label', 'default', 'lookup', 'validation', 'parser')
        model = MetadataType


class DocumentTypeMetadataTypeRelationshipForm(forms.Form):
    RELATIONSHIP_TYPE_NONE = 'none'
    RELATIONSHIP_TYPE_OPTIONAL = 'optional'
    RELATIONSHIP_TYPE_REQUIRED = 'required'
    RELATIONSHIP_CHOICES = (
        (RELATIONSHIP_TYPE_NONE, _('None')),
        (RELATIONSHIP_TYPE_OPTIONAL, _('Optional')),
        (RELATIONSHIP_TYPE_REQUIRED, _('Required')),
    )

    label = forms.CharField(
        label=_('Label'), required=False,
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )
    relationship_type = forms.ChoiceField(
        label=_('Relationship'),
        widget=forms.RadioSelect(), choices=RELATIONSHIP_CHOICES
    )

    def __init__(self, *args, **kwargs):
        self._user = kwargs.pop('_user')
        super(DocumentTypeMetadataTypeRelationshipForm, self).__init__(
            *args, **kwargs
        )

        if 'main_model' in self.initial:
            if self.initial['main_model'] == 'metadata_type':
                self.fields['label'].initial = self.initial['document_type'].label
            else:
                self.fields['label'].initial = self.initial['metadata_type'].label

            self.initial_relationship_type = self.get_relationship_type()
            self.fields['relationship_type'].initial = self.initial_relationship_type

    def get_relationship(self):
        return self.initial['document_type'].metadata.filter(
            metadata_type=self.initial['metadata_type']
        )

    def get_relationship_choices(self):
        return self.initial['document_type'].metadata.filter(
            metadata_type=self.initial['metadata_type']
        )

    def get_relationship_type(self):
        relationship = self.get_relationship()

        if relationship.exists():
            if relationship.get().required:
                return self.RELATIONSHIP_TYPE_REQUIRED
            else:
                return self.RELATIONSHIP_TYPE_OPTIONAL
        else:
            return self.RELATIONSHIP_TYPE_NONE

    def save(self):
        relationship = self.get_relationship()

        if self.cleaned_data['relationship_type'] != self.initial_relationship_type:
            getattr(
                self, 'save_relationship_{}'.format(
                    self.cleaned_data['relationship_type']
                )
            )(relationship=relationship)

    def save_relationship_none(self, relationship):
        relationship.get().delete(_user=self._user)

    def save_relationship_optional(self, relationship):
        if relationship.exists():
            instance = relationship.get()
            instance.required = False
            instance.save(_user=self._user)
        else:
            relationship = DocumentTypeMetadataType(
                document_type=self.initial['document_type'],
                metadata_type=self.initial['metadata_type'],
            )
            relationship.save(_user=self._user)

    def save_relationship_required(self, relationship):
        if relationship.exists():
            instance = relationship.get()
            instance.required = True
            instance.save(_user=self._user)
        else:
            relationship = DocumentTypeMetadataType(
                document_type=self.initial['document_type'],
                metadata_type=self.initial['metadata_type'],
                required=True,
            )
            relationship.save(_user=self._user)


DocumentTypeMetadataTypeRelationshipFormSetBase = formset_factory(
    DocumentTypeMetadataTypeRelationshipForm, extra=0
)


class DocumentTypeMetadataTypeRelationshipFormSet(DocumentTypeMetadataTypeRelationshipFormSetBase):
    def __init__(self, *args, **kwargs):
        _user = kwargs.pop('_user')
        super(DocumentTypeMetadataTypeRelationshipFormSet, self).__init__(*args, **kwargs)
        self.form_kwargs.update({'_user': _user})



from __future__ import unicode_literals

import logging

from django.apps import apps
from django.db.models.signals import post_save, post_delete
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_facet, menu_object, menu_secondary, menu_tools
)
from mayan.apps.navigation.classes import SourceColumn

from .handlers import (
    handler_unverify_key_signatures, handler_verify_key_signatures
)
from .links import (
    link_document_version_all_signature_verify,
    link_document_signature_list,
    link_document_version_signature_delete,
    link_document_version_signature_detached_create,
    link_document_version_signature_embedded_create,
    link_document_version_signature_details,
    link_document_version_signature_download,
    link_document_version_signature_list,
    link_document_version_signature_upload,
)
from .permissions import (
    permission_document_version_sign_detached,
    permission_document_version_sign_embedded,
    permission_document_version_signature_delete,
    permission_document_version_signature_download,
    permission_document_version_signature_upload,
    permission_document_version_signature_view,
)

logger = logging.getLogger(__name__)


class DocumentSignaturesApp(MayanAppConfig):
    app_namespace = 'signatures'
    app_url = 'signatures'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.document_signatures'
    verbose_name = _('Document signatures')

    def ready(self):
        super(DocumentSignaturesApp, self).ready()

        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        DocumentVersion = apps.get_model(
            app_label='documents', model_name='DocumentVersion'
        )

        Key = apps.get_model(
            app_label='django_gpg', model_name='Key'
        )

        DetachedSignature = self.get_model(model_name='DetachedSignature')
        EmbeddedSignature = self.get_model(model_name='EmbeddedSignature')

        SignatureBaseModel = self.get_model(model_name='SignatureBaseModel')

        DocumentVersion.register_post_save_hook(
            order=1, func=EmbeddedSignature.objects.create
        )
        DocumentVersion.register_pre_open_hook(
            order=1, func=EmbeddedSignature.objects.open_signed
        )

        ModelPermission.register(
            model=Document, permissions=(
                permission_document_version_sign_detached,
                permission_document_version_sign_embedded,
                permission_document_version_signature_delete,
                permission_document_version_signature_download,
                permission_document_version_signature_view,
                permission_document_version_signature_upload,
            )
        )
        ModelPermission.register_inheritance(
            model=SignatureBaseModel, related='document_version'
        )
        ModelPermission.register_inheritance(
            model=DetachedSignature, related='document_version'
        )

        SourceColumn(
            source=SignatureBaseModel, label=_('Date'), attribute='date'
        )
        SourceColumn(
            source=SignatureBaseModel, label=_('Key ID'),
            attribute='get_key_id'
        )
        SourceColumn(
            source=SignatureBaseModel, label=_('Signature ID'),
            func=lambda context: context['object'].signature_id or _('None')
        )
        SourceColumn(
            source=SignatureBaseModel, label=_('Type'),
            func=lambda context: SignatureBaseModel.objects.get_subclass(
                pk=context['object'].pk
            ).get_signature_type_display()
        )

        menu_facet.bind_links(
            links=(link_document_signature_list,), sources=(Document,)
        )
        menu_facet.bind_links(
            links=(
                link_document_version_signature_list,
            ), position=9, sources=(DocumentVersion,)
        )

        menu_object.bind_links(
            links=(
                link_document_version_signature_detached_create,
                link_document_version_signature_embedded_create
            ), sources=(DocumentVersion,)
        )
        menu_object.bind_links(
            links=(
                link_document_version_signature_details,
                link_document_version_signature_download,
                link_document_version_signature_delete,
            ), sources=(SignatureBaseModel,)
        )
        menu_secondary.bind_links(
            links=(
                link_document_version_signature_upload,
            ), sources=(DocumentVersion,)
        )
        menu_tools.bind_links(
            links=(link_document_version_all_signature_verify,)
        )

        post_delete.connect(
            dispatch_uid='signatures_handler_unverify_key_signatures',
            receiver=handler_unverify_key_signatures,
            sender=Key
        )
        post_save.connect(
            dispatch_uid='signatures_handler_verify_key_signatures',
            receiver=handler_verify_key_signatures,
            sender=Key
        )



from __future__ import unicode_literals

from django.db import migrations, models
import mayan.apps.documents.models.document_version_models
import mayan.apps.storage.classes


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0047_auto_20180917_0737'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documentversion',
            name='file',
            field=models.FileField(
                storage=mayan.apps.storage.classes.FakeStorageSubclass(),
                upload_to=mayan.apps.documents.models.document_version_models.UUID_FUNCTION,
                verbose_name='File'
            ),
        ),
    ]



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0005_auto_20150708_0327'),
    ]

    operations = [
        migrations.AlterField(
            model_name='source',
            name='label',
            field=models.CharField(max_length=64, verbose_name='Label'),
            preserve_default=True,
        ),
    ]



from __future__ import absolute_import, unicode_literals

from mayan.apps.rest_api import generics

from .models import Key
from .permissions import (
    permission_key_delete, permission_key_upload, permission_key_view
)
from .serializers import KeySerializer


class APIKeyListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the keys.
    post: Upload a new key.
    """
    mayan_object_permissions = {'GET': (permission_key_view,)}
    mayan_view_permissions = {'POST': (permission_key_upload,)}
    queryset = Key.objects.all()
    serializer_class = KeySerializer


class APIKeyView(generics.RetrieveDestroyAPIView):
    """
    delete: Delete the selected key.
    get: Return the details of the selected key.
    """
    mayan_object_permissions = {
        'DELETE': (permission_key_delete,),
        'GET': (permission_key_view,),
    }
    queryset = Key.objects.all()
    serializer_class = KeySerializer



from __future__ import unicode_literals

from django.template.loader import render_to_string

from .icons import icon_fail as default_icon_fail
from .icons import icon_ok as default_icon_ok


class TwoStateWidget(object):
    template_name = 'common/two_state_widget.html'

    def __init__(self, center=False, icon_ok=None, icon_fail=None):
        self.icon_ok = icon_ok or default_icon_ok
        self.icon_fail = icon_fail or default_icon_fail

    def render(self, name=None, value=None):
        return render_to_string(
            template_name=self.template_name, context={
                'icon_ok': self.icon_ok, 'icon_fail': self.icon_fail,
                'value': value
            }
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-12-13 00:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_states', '0016_auto_20191108_2332'),
    ]

    operations = [
        migrations.AlterField(
            model_name='workflow',
            name='label',
            field=models.CharField(help_text='A short text to describe the workflow.', max_length=255, unique=True, verbose_name='Label'),
        ),
        migrations.AlterField(
            model_name='workflowstate',
            name='label',
            field=models.CharField(help_text='A short text to describe the workflow state.', max_length=255, verbose_name='Label'),
        ),
        migrations.AlterField(
            model_name='workflowstateaction',
            name='label',
            field=models.CharField(help_text='A short text describing the action.', max_length=255, verbose_name='Label'),
        ),
        migrations.AlterField(
            model_name='workflowtransition',
            name='label',
            field=models.CharField(help_text='A short text to describe the transition.', max_length=255, verbose_name='Label'),
        ),
    ]



from __future__ import unicode_literals

from django.conf.urls import url
from django.views.i18n import JavaScriptCatalog

from .api_views import (
    APIContentTypeList, APITemplateDetailView, APITemplateListView
)
from .views import (
    AboutView, CurrentUserLocaleProfileDetailsView,
    CurrentUserLocaleProfileEditView, FaviconRedirectView, HomeView,
    LicenseView, ObjectErrorLogEntryListClearView, ObjectErrorLogEntryListView,
    RootView, SetupListView, ToolsListView
)

urlpatterns_error_logs = [
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/errors/$',
        view=ObjectErrorLogEntryListView.as_view(), name='object_error_list'
    ),
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/errors/clear/$',
        view=ObjectErrorLogEntryListClearView.as_view(),
        name='object_error_list_clear'
    ),
]

urlpatterns_user_locale = [
    url(
        regex=r'^user/locale/$',
        view=CurrentUserLocaleProfileDetailsView.as_view(),
        name='current_user_locale_profile_details'
    ),
    url(
        regex=r'^user/locale/edit/$',
        view=CurrentUserLocaleProfileEditView.as_view(),
        name='current_user_locale_profile_edit'
    ),
]

urlpatterns_misc = [
    url(
        regex=r'^favicon\.ico$', view=FaviconRedirectView.as_view()
    ),
    url(
        regex=r'^jsi18n/(?P<packages>\S+?)/$', view=JavaScriptCatalog.as_view(),
        name='javascript_catalog'
    ),
]

urlpatterns = [
    url(regex=r'^$', view=RootView.as_view(), name='root'),
    url(regex=r'^home/$', view=HomeView.as_view(), name='home'),
    url(regex=r'^about/$', view=AboutView.as_view(), name='about_view'),
    url(regex=r'^license/$', view=LicenseView.as_view(), name='license_view'),
    url(regex=r'^setup/$', view=SetupListView.as_view(), name='setup_list'),
    url(regex=r'^tools/$', view=ToolsListView.as_view(), name='tools_list'),
]

urlpatterns.extend(urlpatterns_error_logs)
urlpatterns.extend(urlpatterns_misc)
urlpatterns.extend(urlpatterns_user_locale)

api_urls = [
    url(
        regex=r'^content_types/$', view=APIContentTypeList.as_view(),
        name='content-type-list'
    ),
    url(
        regex=r'^templates/$', view=APITemplateListView.as_view(),
        name='template-list'
    ),
    url(
        regex=r'^templates/(?P<name>[-\w]+)/$',
        view=APITemplateDetailView.as_view(), name='template-detail'
    ),
]
